@@ -34,7 +34,7 @@ static uint opt_no_defaults= 0;
 static uint opt_print_defaults= 0;
 static char *opt_datadir=0, *opt_basedir=0,
             *opt_plugin_dir=0, *opt_plugin_ini=0,
-            *opt_mysqld=0, *opt_my_print_defaults=0, *opt_lc_messages_dir;
+            *opt_mysqld=0, *opt_my_print_defaults=0;
 static char bootstrap[FN_REFLEN];
 
 
@@ -70,8 +70,6 @@ static struct my_option my_long_options[] =
   {"my-print-defaults", 'f', "Path to my_print_defaults executable. "
    "Example: /source/temp11/extra",
     0, 0, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-  {"lc-messages-dir", 'l', "The error messages dir for the server. ",
-    0, 0, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
   {"verbose", 'v',
     "More verbose output; you can use this multiple times to get even more "
     "verbose output.",
@@ -193,7 +191,7 @@ static char *get_value(char *line, const char *item)
     char *s= 0;
 
     s = line + item_len + 1;
-    destination= my_strndup(s, line_len - start, MYF(MY_FAE));
+    destination= my_strndup(PSI_NOT_INSTRUMENTED, s, line_len - start, MYF(MY_FAE));
     destination[line_len - item_len - 2]= 0;
   }
   return destination;
@@ -261,7 +259,7 @@ static int has_spaces(const char *path)
 static char *convert_path(const char *argument)
 {
   /* Convert / to \\ to make Windows paths */
-  char *winfilename= my_strdup(argument, MYF(MY_FAE));
+  char *winfilename= my_strdup(PSI_NOT_INSTRUMENTED, argument, MYF(MY_FAE));
   char *pos, *end;
   size_t length= strlen(argument);
 
@@ -293,7 +291,7 @@ static char *add_quotes(const char *path)
   else
     snprintf(windows_cmd_friendly, sizeof(windows_cmd_friendly),
              "%s", path);
-  return my_strdup(windows_cmd_friendly, MYF(MY_FAE));
+  return my_strdup(PSI_NOT_INSTRUMENTED, windows_cmd_friendly, MYF(MY_FAE));
 }
 #endif
 
@@ -307,7 +305,6 @@ static char *add_quotes(const char *path)
   --basedir
   --plugin-dir
   --plugin-ini
-  --lc-messages-dir
 
   These values are used if the user has not specified a value.
 
@@ -379,26 +376,20 @@ static int get_default_values()
 
       if ((opt_datadir == 0) && ((value= get_value(line, "--datadir"))))
       {
-        opt_datadir= my_strdup(value, MYF(MY_FAE));
+        opt_datadir= my_strdup(PSI_NOT_INSTRUMENTED, value, MYF(MY_FAE));
       }
       if ((opt_basedir == 0) && ((value= get_value(line, "--basedir"))))
       {
-        opt_basedir= my_strdup(value, MYF(MY_FAE));
+        opt_basedir= my_strdup(PSI_NOT_INSTRUMENTED, value, MYF(MY_FAE));
       }
-      if ((opt_plugin_dir == 0) && ((value= get_value(line, "--plugin_dir")) ||
-          (value= get_value(line, "--plugin-dir"))))
+      if ((opt_plugin_dir == 0) && ((value= get_value(line, "--plugin_dir"))))
       {
-        opt_plugin_dir= my_strdup(value, MYF(MY_FAE));
+        opt_plugin_dir= my_strdup(PSI_NOT_INSTRUMENTED, value, MYF(MY_FAE));
       }
-      if ((opt_lc_messages_dir == 0) &&
-          ((value= get_value(line, "--lc_messages_dir")) ||
-          (value= get_value(line, "--lc_messages-dir")) ||
-          (value= get_value(line, "--lc-messages_dir")) ||
-          (value= get_value(line, "--lc-messages-dir"))))
+      if ((opt_plugin_ini == 0) && ((value= get_value(line, "--plugin_ini"))))
       {
-        opt_lc_messages_dir= my_strdup(value, MYF(MY_FAE));
+        opt_plugin_ini= my_strdup(PSI_NOT_INSTRUMENTED, value, MYF(MY_FAE));
       }
-
     }
   }
 exit:
@@ -438,7 +429,6 @@ static void usage(void)
   --basedir
   --plugin-dir
   --plugin-ini
-  --lc-messages-dir
 
 */
 
@@ -471,10 +461,6 @@ static void print_default_values(void)
   {
     printf("--my_print_defaults=%s ", opt_my_print_defaults);
   }
-  if (opt_lc_messages_dir)
-  {
-    printf("--lc_messages_dir=%s ", opt_lc_messages_dir);
-  }
   printf("\n");
 }
 
@@ -488,11 +474,10 @@ static void print_default_values(void)
 */
 
 static my_bool
-get_one_option(int optid,
-               const struct my_option *opt __attribute__((unused)),
-               char *argument)
+get_one_option(const struct my_option *opt,
+               char *argument, const char *filename __attribute__((unused)))
 {
-  switch(optid) {
+  switch(opt->id) {
   case 'n':
     opt_no_defaults++;
     break;
@@ -512,27 +497,23 @@ get_one_option(int optid,
     usage();
     exit(0);
   case 'd':
-    opt_datadir= my_strdup(argument, MYF(MY_FAE));
+    opt_datadir= my_strdup(PSI_NOT_INSTRUMENTED, argument, MYF(MY_FAE));
     break;
   case 'b':
-    opt_basedir= my_strdup(argument, MYF(MY_FAE));
+    opt_basedir= my_strdup(PSI_NOT_INSTRUMENTED, argument, MYF(MY_FAE));
     break;
   case 'p':
-    opt_plugin_dir= my_strdup(argument, MYF(MY_FAE));
+    opt_plugin_dir= my_strdup(PSI_NOT_INSTRUMENTED, argument, MYF(MY_FAE));
     break;
   case 'i':
-    opt_plugin_ini= my_strdup(argument, MYF(MY_FAE));
+    opt_plugin_ini= my_strdup(PSI_NOT_INSTRUMENTED, argument, MYF(MY_FAE));
     break;
   case 'm':
-    opt_mysqld= my_strdup(argument, MYF(MY_FAE));
+    opt_mysqld= my_strdup(PSI_NOT_INSTRUMENTED, argument, MYF(MY_FAE));
     break;
   case 'f':
-    opt_my_print_defaults= my_strdup(argument, MYF(MY_FAE));
+    opt_my_print_defaults= my_strdup(PSI_NOT_INSTRUMENTED, argument, MYF(MY_FAE));
     break;
-  case 'l':
-    opt_lc_messages_dir= my_strdup(argument, MYF(MY_FAE));
-    break;
-
   }
   return 0;
 }
@@ -639,7 +620,7 @@ static int load_plugin_data(char *plugin_name, char *config_file)
   if (opt_plugin_ini == 0)
   {
     fn_format(path, config_file, opt_plugin_dir, "", MYF(0));
-    opt_plugin_ini= my_strdup(path, MYF(MY_FAE));
+    opt_plugin_ini= my_strdup(PSI_NOT_INSTRUMENTED, path, MYF(MY_FAE));
   }
   if (!file_exists(opt_plugin_ini))
   {
@@ -655,7 +636,7 @@ static int load_plugin_data(char *plugin_name, char *config_file)
   }
 
   /* save name */
-  plugin_data.name= my_strdup(plugin_name, MYF(MY_WME));
+  plugin_data.name= my_strdup(PSI_NOT_INSTRUMENTED, plugin_name, MYF(MY_WME));
 
   /* Read plugin components */
   while (i < 16)
@@ -685,14 +666,14 @@ static int load_plugin_data(char *plugin_name, char *config_file)
       /* Add proper file extension for soname */
       strcat(line, FN_SOEXT);
       /* save so_name */
-      plugin_data.so_name= my_strdup(line, MYF(MY_WME|MY_ZEROFILL));
+      plugin_data.so_name= my_strdup(PSI_NOT_INSTRUMENTED, line, MYF(MY_WME|MY_ZEROFILL));
       i++;
     }
     else
     {
       if (strlen(line) > 0)
       {
-        plugin_data.components[i]= my_strdup(line, MYF(MY_WME));
+        plugin_data.components[i]= my_strdup(PSI_NOT_INSTRUMENTED, line, MYF(MY_WME));
         i++;
       }
       else
@@ -758,21 +739,21 @@ static int check_options(int argc, char **argv, char *operation)
     else if ((strncasecmp(argv[i], basedir_prefix, basedir_len) == 0) &&
              !opt_basedir)
     {
-      opt_basedir= my_strndup(argv[i]+basedir_len,
+      opt_basedir= my_strndup(PSI_NOT_INSTRUMENTED, argv[i]+basedir_len,
                               strlen(argv[i])-basedir_len, MYF(MY_FAE));
       num_found++;
     }
     else if ((strncasecmp(argv[i], datadir_prefix, datadir_len) == 0) &&
              !opt_datadir)
     {
-      opt_datadir= my_strndup(argv[i]+datadir_len,
+      opt_datadir= my_strndup(PSI_NOT_INSTRUMENTED, argv[i]+datadir_len,
                               strlen(argv[i])-datadir_len, MYF(MY_FAE));
       num_found++;
     }
     else if ((strncasecmp(argv[i], plugin_dir_prefix, plugin_dir_len) == 0) &&
              !opt_plugin_dir)
     {
-      opt_plugin_dir= my_strndup(argv[i]+plugin_dir_len,
+      opt_plugin_dir= my_strndup(PSI_NOT_INSTRUMENTED, argv[i]+plugin_dir_len,
                                  strlen(argv[i])-plugin_dir_len, MYF(MY_FAE));
       num_found++;
     }
@@ -885,7 +866,7 @@ static int process_options(int argc, char *argv[], char *operation)
 #endif
       buff[sizeof(buff) - 1]= 0;
       my_free(opt_basedir);
-      opt_basedir= my_strdup(buff, MYF(MY_FAE));
+      opt_basedir= my_strdup(PSI_NOT_INSTRUMENTED, buff, MYF(MY_FAE));
     }
   }
   
@@ -918,8 +899,6 @@ static int process_options(int argc, char *argv[], char *operation)
     printf("# plugin_dir = %s\n", opt_plugin_dir);
     printf("#    datadir = %s\n", opt_datadir);
     printf("# plugin_ini = %s\n", opt_plugin_ini);
-    if (opt_lc_messages_dir != 0)
-      printf("# lc_messages_dir = %s\n", opt_lc_messages_dir);
   }
 
 exit:
@@ -977,12 +956,6 @@ static int check_access()
             opt_my_print_defaults);
     goto exit;
   }
-  if (opt_lc_messages_dir && (error= my_access(opt_lc_messages_dir, F_OK)))
-  {
-    fprintf(stderr, "ERROR: Cannot access lc-messages-dir path '%s'.\n",
-            opt_lc_messages_dir);
-    goto exit;
-  }
 
 exit:
   return error;
@@ -1212,33 +1185,18 @@ static int bootstrap_server(char *server_path, char *bootstrap_file)
     verbose_str= "";
   if (has_spaces(opt_datadir) || has_spaces(opt_basedir) ||
       has_spaces(bootstrap_file))
-  {
-    if (opt_lc_messages_dir != NULL)
-      format_str= "\"%s %s --bootstrap --datadir=%s --basedir=%s --lc-messages-dir=%s <%s\"";
-    else
-    format_str= "\"%s %s --bootstrap --datadir=%s --basedir=%s <%s\"";
-  }
-  else
-  {
-    if (opt_lc_messages_dir != NULL)
-      format_str= "\"%s %s --bootstrap --datadir=%s --basedir=%s --lc-messages-dir=%s <%s\"";
-    else
-      format_str= "%s %s --bootstrap --datadir=%s --basedir=%s <%s";
-  }
+    format_str= "\"%s %s --bootstrap --datadir=%s --basedir=%s < %s\"";
+  else 
+    format_str= "%s %s --bootstrap --datadir=%s --basedir=%s < %s";
+
   snprintf(bootstrap_cmd, sizeof(bootstrap_cmd), format_str,
            add_quotes(convert_path(server_path)), verbose_str,
            add_quotes(opt_datadir), add_quotes(opt_basedir),
            add_quotes(bootstrap_file));
 #else
-  if (opt_lc_messages_dir != NULL)
-    snprintf(bootstrap_cmd, sizeof(bootstrap_cmd),
-             "%s --no-defaults --bootstrap --datadir=%s --basedir=%s --lc-messages-dir=%s"
-             " <%s", server_path, opt_datadir, opt_basedir, opt_lc_messages_dir, bootstrap_file);
-  else
-    snprintf(bootstrap_cmd, sizeof(bootstrap_cmd),
-             "%s --no-defaults --bootstrap --datadir=%s --basedir=%s"
-             " <%s", server_path, opt_datadir, opt_basedir, bootstrap_file);
-
+  snprintf(bootstrap_cmd, sizeof(bootstrap_cmd),
+           "%s --no-defaults --bootstrap --datadir=%s --basedir=%s"
+           " < %s", server_path, opt_datadir, opt_basedir, bootstrap_file);
 #endif
 
   /* Execute the command */
