@@ -466,14 +466,13 @@ struct datafile_cur_t {
 	char		abs_path[FN_REFLEN];
 	MY_STAT		statinfo;
 	uint		thread_n;
-	byte*		orig_buf;
 	byte*		buf;
 	size_t		buf_size;
 	size_t		buf_read;
 	size_t		buf_offset;
 
 	explicit datafile_cur_t(const char* filename = NULL) :
-		file(), thread_n(0), orig_buf(NULL), buf(NULL), buf_size(0),
+		file(), thread_n(0), buf(NULL), buf_size(0),
 		buf_read(0), buf_offset(0)
 	{
 		memset(rel_path, 0, sizeof rel_path);
@@ -864,14 +863,21 @@ datafile_rsync_backup(const char *filepath, bool save_to_list, FILE *f)
 	return(true);
 }
 
-bool backup_file_print_buf(const char *filename, const char *buf, int buf_len)
+
+static
+bool
+backup_file_vprintf(const char *filename, const char *fmt, va_list ap)
 {
 	ds_file_t	*dstfile	= NULL;
 	MY_STAT		 stat;			/* unused for now */
+	char		*buf		= 0;
+	int		 buf_len;
 	const char	*action;
 
 	memset(&stat, 0, sizeof(stat));
 
+	buf_len = vasprintf(&buf, fmt, ap);
+
 	stat.st_size = buf_len;
 	stat.st_mtime = my_time(0);
 
@@ -895,6 +901,7 @@ bool backup_file_print_buf(const char *filename, const char *buf, int buf_len)
 
 	/* close */
 	msg("        ...done");
+	free(buf);
 
 	if (ds_close(dstfile)) {
 		goto error_close;
@@ -903,6 +910,7 @@ bool backup_file_print_buf(const char *filename, const char *buf, int buf_len)
 	return(true);
 
 error:
+	free(buf);
 	if (dstfile != NULL) {
 		ds_close(dstfile);
 	}
@@ -910,22 +918,9 @@ error:
 error_close:
 	msg("Error: backup file failed.");
 	return(false); /*ERROR*/
-
-	return true;
-};
-
-static
-bool
-backup_file_vprintf(const char *filename, const char *fmt, va_list ap)
-{
-	char		*buf		= 0;
-	int		 buf_len;
-	buf_len = vasprintf(&buf, fmt, ap);
-	bool result = backup_file_print_buf(filename, buf, buf_len);
-	free(buf);
-	return result;
 }
 
+
 bool
 backup_file_printf(const char *filename, const char *fmt, ...)
 {
@@ -1032,16 +1027,16 @@ static int fix_win_file_permissions(const char *file)
 	ea.grfInheritance = CONTAINER_INHERIT_ACE | OBJECT_INHERIT_ACE;
 	ea.Trustee.TrusteeType = TRUSTEE_IS_UNKNOWN;
 	ACL* pNewDACL = 0;
-	SetEntriesInAcl(1, &ea, pOldDACL, &pNewDACL);
-	if (pNewDACL)
+	DWORD err = SetEntriesInAcl(1, &ea, pOldDACL, &pNewDACL);
+	if (!err)
 	{
+		DBUG_ASSERT(pNewDACL);
 		SetSecurityInfo(hFile, SE_FILE_OBJECT, DACL_SECURITY_INFORMATION, NULL, NULL,
 			pNewDACL, NULL);
+		LocalFree((HLOCAL)pNewDACL);
 	}
 	if (pSD != NULL)
 		LocalFree((HLOCAL)pSD);
-	if (pNewDACL != NULL)
-		LocalFree((HLOCAL)pNewDACL);
 	CloseHandle(hFile);
 	return 0;
 }
@@ -1062,7 +1057,6 @@ copy_file(ds_ctxt_t *datasink,
 	ds_file_t		*dstfile = NULL;
 	datafile_cur_t		 cursor;
 	xb_fil_cur_result_t	 res;
-	DBUG_ASSERT(datasink->datasink->remove);
 	const char	*dst_path =
 		(xtrabackup_copy_back || xtrabackup_move_back)?
 		dst_file_path : trim_dotslash(dst_file_path);
@@ -1088,7 +1082,6 @@ copy_file(ds_ctxt_t *datasink,
 		if (ds_write(dstfile, cursor.buf, cursor.buf_read)) {
 			goto error;
 		}
-		DBUG_EXECUTE_IF("copy_file_error", errno=ENOSPC;goto error;);
 	}
 
 	if (res == XB_FIL_CUR_ERROR) {
@@ -1110,7 +1103,6 @@ copy_file(ds_ctxt_t *datasink,
 error:
 	datafile_close(&cursor);
 	if (dstfile != NULL) {
-		datasink->datasink->remove(dstfile->path);
 		ds_close(dstfile);
 	}
 
@@ -1155,18 +1147,17 @@ move_file(ds_ctxt_t *datasink,
 
 	if (my_rename(src_file_path, dst_file_path_abs, MYF(0)) != 0) {
 		if (my_errno == EXDEV) {
-			/* Fallback to copy/unlink */
-			if(!copy_file(datasink, src_file_path,
-					dst_file_path, thread_n))
-					return false;
+			bool ret;
+			ret = copy_file(datasink, src_file_path,
+					dst_file_path, thread_n);
 			msg(thread_n,"Removing %s", src_file_path);
 			if (unlink(src_file_path) != 0) {
 				my_strerror(errbuf, sizeof(errbuf), errno);
-				msg("Warning: unlink %s failed: %s",
+				msg("Error: unlink %s failed: %s",
 					src_file_path,
 					errbuf);
 			}
-			return true;
+			return(ret);
 		}
 		my_strerror(errbuf, sizeof(errbuf), my_errno);
 		msg("Can not move file %s to %s: %s",
@@ -1450,9 +1441,9 @@ out:
 	return(ret);
 }
 
-void backup_fix_ddl(CorruptedPages &);
+void backup_fix_ddl(void);
 
-lsn_t get_current_lsn(MYSQL *connection)
+static lsn_t get_current_lsn(MYSQL *connection)
 {
 	static const char lsn_prefix[] = "\nLog sequence number ";
 	lsn_t lsn = 0;
@@ -1475,7 +1466,7 @@ lsn_t get_current_lsn(MYSQL *connection)
 lsn_t server_lsn_after_lock;
 extern void backup_wait_for_lsn(lsn_t lsn);
 /** Start --backup */
-bool backup_start(CorruptedPages &corrupted_pages)
+bool backup_start()
 {
 	if (!opt_no_lock) {
 		if (opt_safe_slave_backup) {
@@ -1510,7 +1501,7 @@ bool backup_start(CorruptedPages &corrupted_pages)
 
 	msg("Waiting for log copy thread to read lsn %llu", (ulonglong)server_lsn_after_lock);
 	backup_wait_for_lsn(server_lsn_after_lock);
-	backup_fix_ddl(corrupted_pages);
+	backup_fix_ddl();
 
 	// There is no need to stop slave thread before coping non-Innodb data when
 	// --no-lock option is used because --no-lock option requires that no DDL or
@@ -1790,7 +1781,7 @@ apply_log_finish()
 bool
 copy_back()
 {
-	bool ret = false;
+	bool ret;
 	datadir_iter_t *it = NULL;
 	datadir_node_t node;
 	char *dst_dir;
@@ -1872,24 +1863,25 @@ copy_back()
 	dst_dir = (srv_log_group_home_dir && *srv_log_group_home_dir)
 		? srv_log_group_home_dir : mysql_data_home;
 
-	/* --backup generates a single ib_logfile0, which we must copy
+	/* --backup generates a single LOG_FILE_NAME, which we must copy
 	if it exists. */
 
 	ds_data = ds_create(dst_dir, DS_TYPE_LOCAL);
 	MY_STAT stat_arg;
-	if (!my_stat("ib_logfile0", &stat_arg, MYF(0)) || !stat_arg.st_size) {
+	if (!my_stat(LOG_FILE_NAME, &stat_arg, MYF(0)) || !stat_arg.st_size) {
 		/* After completed --prepare, redo log files are redundant.
 		We must delete any redo logs at the destination, so that
 		the database will not jump to a different log sequence number
 		(LSN). */
 
-		for (uint i = 0; i <= SRV_N_LOG_FILES_MAX + 1; i++) {
-			char filename[FN_REFLEN];
-			snprintf(filename, sizeof filename, "%s/ib_logfile%u",
-				 dst_dir, i);
-			unlink(filename);
-		}
-	} else if (!(ret = copy_or_move_file("ib_logfile0", "ib_logfile0",
+		char filename[FN_REFLEN];
+		snprintf(filename, sizeof filename, "%s/%s0", dst_dir,
+			 LOG_FILE_NAME_PREFIX);
+		unlink(filename);
+		snprintf(filename, sizeof filename, "%s/%s101", dst_dir,
+			 LOG_FILE_NAME_PREFIX);
+		unlink(filename);
+	} else if (!(ret = copy_or_move_file(LOG_FILE_NAME, LOG_FILE_NAME,
 					     dst_dir, 1))) {
 		goto cleanup;
 	}
@@ -1981,7 +1973,7 @@ copy_back()
 		}
 
 		/* skip the redo log (it was already copied) */
-		if (!strcmp(filename, "ib_logfile0")) {
+		if (!strcmp(filename, LOG_FILE_NAME)) {
 			continue;
 		}
 
