@@ -626,10 +626,8 @@ sub make_combinations($$@)
   {
     # Skip all other combinations if the values they change
     # are already fixed in master_opt or slave_opt
-    # (empty combinations are not considered a subset of anything)
-    if (@{$comb->{comb_opt}} &&
-        My::Options::is_subset($test->{master_opt}, $comb->{comb_opt}) &&
-        My::Options::is_subset($test->{slave_opt}, $comb->{comb_opt}) ){
+    if (My::Options::is_set($test->{master_opt}, $comb->{comb_opt}) &&
+        My::Options::is_set($test->{slave_opt}, $comb->{comb_opt}) ){
 
       $test_combs->{$comb->{name}} = 2;
 
