@@ -1,5 +1,5 @@
 /* Copyright (c) 2000, 2010, Oracle and/or its affiliates.
-   Copyright (c) 2011, 2013, Monty Program Ab.
+   Copyright (c) 2011, 2020, MariaDB Corporation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@ my_hash_value_type my_hash_sort(CHARSET_INFO *cs, const uchar *key,
                                 size_t length)
 {
   ulong nr1= 1, nr2= 4;
-  cs->coll->hash_sort(cs, (uchar*) key, length, &nr1, &nr2);
+  my_ci_hash_sort(cs, (uchar*) key, length, &nr1, &nr2);
   return (my_hash_value_type) nr1;
 }
 
@@ -76,14 +76,14 @@ my_hash_value_type my_hash_sort(CHARSET_INFO *cs, const uchar *key,
     @retval 1 failure
 */
 my_bool
-my_hash_init2(HASH *hash, uint growth_size, CHARSET_INFO *charset,
-              ulong size, size_t key_offset, size_t key_length,
-              my_hash_get_key get_key,
+my_hash_init2(PSI_memory_key psi_key, HASH *hash, uint growth_size,
+              CHARSET_INFO *charset, ulong size, size_t key_offset,
+              size_t key_length, my_hash_get_key get_key,
               my_hash_function hash_function,
               void (*free_element)(void*), uint flags)
 {
   my_bool res;
-  DBUG_ENTER("my_hash_init");
+  DBUG_ENTER("my_hash_init2");
   DBUG_PRINT("enter",("hash:%p  size: %u", hash, (uint) size));
 
   hash->records=0;
@@ -95,7 +95,7 @@ my_hash_init2(HASH *hash, uint growth_size, CHARSET_INFO *charset,
   hash->free=free_element;
   hash->flags=flags;
   hash->charset=charset;
-  res= init_dynamic_array2(&hash->array, sizeof(HASH_LINK), NULL, size,
+  res= init_dynamic_array2(psi_key, &hash->array, sizeof(HASH_LINK), NULL, size,
                            growth_size, MYF((flags & HASH_THREAD_SPECIFIC ?
                                              MY_THREAD_SPECIFIC : 0)));
   DBUG_RETURN(res);
@@ -895,7 +895,7 @@ int main(int argc __attribute__((unused)),char **argv __attribute__((unused)))
   DBUG_PUSH("d:t:O,/tmp/test_hash.trace");
 
   printf("my_hash_init\n");
-  if (my_hash_init2(&hash_test, 100, &my_charset_bin, 20,
+  if (my_hash_init2(PSI_INSTRUMENT_ME, &hash_test, 100, &my_charset_bin, 20,
                     0, 0, (my_hash_get_key) test_get_key, 0, 0, HASH_UNIQUE))
   {
     fprintf(stderr, "hash init failed\n");
