@@ -73,14 +73,12 @@ usage () {
         cat <<EOF
 Usage: $0 [OPTIONS]
   --no-defaults              Don't read the system defaults file
+  --core-file-size=LIMIT     Limit core files to the specified size
   --defaults-file=FILE       Use the specified defaults file
   --defaults-extra-file=FILE Also use defaults from the specified file
-  --defaults-group-suffix=X  Additionally read default groups with X appended
-                             as a suffix
   --ledir=DIRECTORY          Look for mysqld in the specified directory
   --open-files-limit=LIMIT   Limit the number of open files
   --crash-script=FILE        Script to call when mysqld crashes
-  --core-file-size=LIMIT     Limit core files to the specified size
   --timezone=TZ              Set the system timezone
   --malloc-lib=LIB           Preload shared library LIB if available
   --mysqld=FILE              Use the specified file as mysqld
@@ -128,7 +126,7 @@ log_generic () {
   case $logging in
     init) ;;  # Just echo the message, don't save it anywhere
     file)
-      if [ "$helper_exist" -eq "0" ]; then
+      if [ -n "$helper" ]; then
         echo "$msg" | "$helper" "$user" log "$err_log"
       fi
     ;;
@@ -152,7 +150,7 @@ eval_log_error () {
   local cmd="$1"
   case $logging in
     file)
-     if [ "$helper_exist" -eq "0" ]; then
+     if [ -n "$helper" ]; then
         cmd="$cmd 2>&1 | "`shell_quote_string "$helper"`" $user log "`shell_quote_string "$err_log"`
      fi
      ;;
@@ -439,10 +437,25 @@ mysqld_ld_preload_text() {
   echo "$text"
 }
 
+
+mysql_config=
+get_mysql_config() {
+  if [ -z "$mysql_config" ]; then
+    mysql_config=`echo "$0" | sed 's,/[^/][^/]*$,/mysql_config,'`
+    if [ ! -x "$mysql_config" ]; then
+      log_error "Can not run mysql_config $@ from '$mysql_config'"
+      exit 1
+    fi
+  fi
+
+  "$mysql_config" "$@"
+}
+
+
 # set_malloc_lib LIB
 # - If LIB is empty, do nothing and return
-# - If LIB starts with 'tcmalloc' or 'jemalloc', look for the shared library
-#   using `ldconfig`.
+# - If LIB starts with 'tcmalloc' or 'jemalloc', look for the shared library in
+#   /usr/lib, /usr/lib64 and then pkglibdir.
 #   tcmalloc is part of the Google perftools project.
 # - If LIB is an absolute path, assume it is a malloc shared library
 #
@@ -450,29 +463,28 @@ mysqld_ld_preload_text() {
 # running mysqld.  See ld.so for details.
 set_malloc_lib() {
   malloc_lib="$1"
+
   if expr "$malloc_lib" : "\(tcmalloc\|jemalloc\)" > /dev/null ; then
-    export PATH=$PATH:/sbin
-    if ! command -v ldconfig > /dev/null 2>&1
-    then
-      log_error "ldconfig command not found, required for ldconfig -p"
-      exit 1
-    fi
-    # format from ldconfig:
-    # "libjemalloc.so.1 (libc6,x86-64) => /usr/lib/x86_64-linux-gnu/libjemalloc.so.1"
-    libmalloc_path="$(ldconfig -p | sed -n "/lib${malloc_lib}/p" | cut -d '>' -f2)"
-
-    if [ -z "$libmalloc_path" ]; then
-      log_error "no shared library for lib$malloc_lib.so.[0-9] found."
-      exit 1
-    fi
-
-    for f in $libmalloc_path; do
-      if [ -f "$f" ]; then
-        malloc_lib=$f # get the first path if many
-        break
-      fi
+    pkglibdir=`get_mysql_config --variable=pkglibdir`
+    where=''
+    # This list is kept intentionally simple.  Simply set --malloc-lib
+    # to a full path if another location is desired.
+    for libdir in /usr/lib /usr/lib64 "$pkglibdir" "$pkglibdir/mysql"; do
+       tmp=`echo "$libdir/lib$malloc_lib.so".[0-9]`
+       where="$where $libdir"
+       # log_notice "DEBUG: Checking for malloc lib '$tmp'"
+       [ -r "$tmp" ] || continue
+       malloc_lib="$tmp"
+       where=''
+       break
     done
+
+    if [ -n "$where" ]; then
+      log_error "no shared library for lib$malloc_lib.so.[0-9] found in$where"
+      exit 1
+    fi
   fi
+
   # Allow --malloc-lib='' to override other settings
   [ -z  "$malloc_lib" ] && return
 
@@ -489,6 +501,7 @@ set_malloc_lib() {
       exit 1
       ;;
   esac
+
   add_mysqld_ld_preload "$malloc_lib"
 }
 
@@ -505,25 +518,25 @@ then
   # BASEDIR is already overridden on command line.  Do not re-set.
 
   # Use BASEDIR to discover le.
-  if test -x "$MY_BASEDIR_VERSION/libexec/mysqld"
+  if test -x "$MY_BASEDIR_VERSION/libexec/mariadbd"
   then
     ledir="$MY_BASEDIR_VERSION/libexec"
-  elif test -x "$MY_BASEDIR_VERSION/sbin/mysqld"
+  elif test -x "$MY_BASEDIR_VERSION/sbin/mariadbd"
   then
     ledir="$MY_BASEDIR_VERSION/sbin"
   else
     ledir="$MY_BASEDIR_VERSION/bin"
   fi
-elif test -x "$MY_PWD/bin/mysqld"
+elif test -x "$MY_PWD/bin/mariadbd"
 then
   MY_BASEDIR_VERSION="$MY_PWD"		# Where bin, share and data are
   ledir="$MY_PWD/bin"			# Where mysqld is
 # Check for the directories we would expect from a source install
-elif test -x "$MY_PWD/libexec/mysqld"
+elif test -x "$MY_PWD/libexec/mariadbd"
 then
   MY_BASEDIR_VERSION="$MY_PWD"		# Where libexec, share and var are
   ledir="$MY_PWD/libexec"		# Where mysqld is
-elif test -x "$MY_PWD/sbin/mysqld"
+elif test -x "$MY_PWD/sbin/mariadbd"
 then
   MY_BASEDIR_VERSION="$MY_PWD"		# Where sbin, share and var are
   ledir="$MY_PWD/sbin"			# Where mysqld is
@@ -535,9 +548,10 @@ fi
 
 helper=`find_in_bin mysqld_safe_helper`
 print_defaults=`find_in_bin my_print_defaults`
+
 # Check if helper exists
-command -v $helper --help >/dev/null 2>&1
-helper_exist=$?
+$helper --help >/dev/null 2>&1 || helper=""
+
 #
 # Second, try to find the data directory
 #
@@ -556,7 +570,7 @@ else
 fi
 
 if test -z "$MYSQL_HOME"
-then 
+then
   if test -r "$DATADIR/my.cnf"
   then
     log_error "WARNING: Found $DATADIR/my.cnf
@@ -645,7 +659,7 @@ then
 
     # mysqld does not add ".err" to "--log-error=foo."; it considers a
     # trailing "." as an extension
-    
+
     if expr "$err_log" : '.*\.[^/]*$' > /dev/null
     then
         :
@@ -727,10 +741,10 @@ then
   chmod 755 $mysql_unix_port_dir
 fi
 
-# If the user doesn't specify a binary, we assume name "mysqld"
+# If the user doesn't specify a binary, we assume name "mariadbd"
 if test -z "$MYSQLD"
 then
-  MYSQLD=mysqld
+  MYSQLD=mariadbd
 fi
 
 if test ! -x "$ledir/$MYSQLD"
@@ -944,6 +958,7 @@ fi
 
 # Avoid 'nohup: ignoring input' warning
 test -n "$NOHUP_NICENESS" && cmd="$cmd < /dev/null"
+
 log_notice "Starting $MYSQLD daemon with databases from $DATADIR"
 
 # variable to track the current number of "fast" (a.k.a. subsecond) restarts
@@ -1035,8 +1050,8 @@ do
     log_notice "Number of processes running now: $numofproces"
     I=1
     while test "$I" -le "$numofproces"
-    do 
-      PROC=`ps xaww | grep "$ledir/$MYSQLD\>" | grep -v "grep" | grep "pid-file=$pid_file" | sed -n '$p'` 
+    do
+      PROC=`ps xaww | grep "$ledir/$MYSQLD\>" | grep -v "grep" | grep "pid-file=$pid_file" | sed -n '$p'`
 
       for T in $PROC
       do
