@@ -35,7 +35,7 @@
 #include "sql_table.h"
 
 static int count_relay_log_space(Relay_log_info* rli);
-
+bool xa_trans_force_rollback(THD *thd);
 /**
    Current replication state (hash of last GTID executed, per replication
    domain).
@@ -46,8 +46,8 @@ gtid_waiting rpl_global_gtid_waiting;
 
 const char *const Relay_log_info::state_delaying_string = "Waiting until MASTER_DELAY seconds after master executed event";
 
-Relay_log_info::Relay_log_info(bool is_slave_recovery)
-  :Slave_reporting_capability("SQL"),
+Relay_log_info::Relay_log_info(bool is_slave_recovery, const char* thread_name)
+  :Slave_reporting_capability(thread_name),
    replicate_same_server_id(::replicate_same_server_id),
    info_fd(-1), cur_log_fd(-1), relay_log(&sync_relaylog_period),
    sync_counter(0), is_relay_log_recovery(is_slave_recovery),
@@ -74,7 +74,9 @@ Relay_log_info::Relay_log_info(bool is_slave_recovery)
                          key_RELAYLOG_COND_relay_log_updated,
                          key_RELAYLOG_COND_bin_log_updated,
                          key_file_relaylog,
+                         key_file_relaylog_cache,
                          key_file_relaylog_index,
+                         key_file_relaylog_index_cache,
                          key_RELAYLOG_COND_queue_busy,
                          key_LOCK_relaylog_end_pos);
 #endif
@@ -207,8 +209,8 @@ a file name for --relay-log-index option", opt_relaylog_index_name);
       */
       sql_print_warning("Neither --relay-log nor --relay-log-index were used;"
                         " so replication "
-                        "may break when this MariaDB server acts as a "
-                        "replica and has its hostname changed. Please "
+                        "may break when this MySQL server acts as a "
+                        "slave and has his hostname changed!! Please "
                         "use '--log-basename=#' or '--relay-log=%s' to avoid "
                         "this problem.", ln);
       name_warning_sent= 1;
@@ -238,7 +240,7 @@ a file name for --relay-log-index option", opt_relaylog_index_name);
     */
     mysql_mutex_lock(log_lock);
     if (relay_log.open_index_file(buf_relaylog_index_name, ln, TRUE) ||
-        relay_log.open(ln, LOG_BIN, 0, 0, SEQ_READ_APPEND,
+        relay_log.open(ln, 0, 0, SEQ_READ_APPEND,
                        (ulong)max_relay_log_size, 1, TRUE))
     {
       mysql_mutex_unlock(log_lock);
@@ -266,7 +268,7 @@ a file name for --relay-log-index option", opt_relaylog_index_name);
       msg= current_thd->get_stmt_da()->message();
       goto err;
     }
-    if (init_io_cache(&info_file, info_fd, IO_SIZE*2, READ_CACHE, 0L,0,
+    if (init_io_cache(&info_file, info_fd, LOG_BIN_IO_SIZE, READ_CACHE, 0L,0,
                       MYF(MY_WME)))
     {
       sql_print_error("Failed to create a cache on relay log info file '%s'",
@@ -301,7 +303,7 @@ Failed to open the existing relay log info file '%s' (errno %d)",
         error= 1;
       }
       else if (init_io_cache(&info_file, info_fd,
-                             IO_SIZE*2, READ_CACHE, 0L, 0, MYF(MY_WME)))
+                             LOG_BIN_IO_SIZE, READ_CACHE, 0L, 0, MYF(MY_WME)))
       {
         sql_print_error("Failed to create a cache on relay log info file '%s'",
                         fname);
@@ -459,7 +461,7 @@ static inline int add_relay_log(Relay_log_info* rli,LOG_INFO* linfo)
     DBUG_RETURN(1);
   }
   rli->log_space_total += s.st_size;
-  DBUG_PRINT("info",("log_space_total: %llu", uint64(rli->log_space_total)));
+  DBUG_PRINT("info",("log_space_total: %llu", rli->log_space_total));
   DBUG_RETURN(0);
 }
 
@@ -991,7 +993,7 @@ void Relay_log_info::inc_group_relay_log_pos(ulonglong log_pos,
   if (rgi->is_parallel_exec)
   {
     /* In case of parallel replication, do not update the position backwards. */
-    int cmp= compare_log_name(group_relay_log_name, rgi->event_relay_log_name);
+    int cmp= strcmp(group_relay_log_name, rgi->event_relay_log_name);
     if (cmp < 0)
     {
       group_relay_log_pos= rgi->future_event_relay_log_pos;
@@ -1003,7 +1005,7 @@ void Relay_log_info::inc_group_relay_log_pos(ulonglong log_pos,
       In the parallel case we need to update the master_log_name here, rather
       than in Rotate_log_event::do_update_pos().
     */
-    cmp= compare_log_name(group_master_log_name, rgi->future_event_master_log_name);
+    cmp= strcmp(group_master_log_name, rgi->future_event_master_log_name);
     if (cmp <= 0)
     {
       if (cmp < 0)
@@ -1180,7 +1182,7 @@ int purge_relay_logs(Relay_log_info* rli, THD *thd, bool just_reset,
         DBUG_RETURN(1);
       }
       mysql_mutex_lock(rli->relay_log.get_log_lock());
-      if (rli->relay_log.open(ln, LOG_BIN, 0, 0, SEQ_READ_APPEND,
+      if (rli->relay_log.open(ln, 0, 0, SEQ_READ_APPEND,
                              (ulong)(rli->max_relay_log_size ? rli->max_relay_log_size :
                               max_binlog_size), 1, TRUE))
       {
@@ -1252,7 +1254,7 @@ int purge_relay_logs(Relay_log_info* rli, THD *thd, bool just_reset,
     mysql_mutex_unlock(rli->relay_log.get_log_lock());
   }
 err:
-  DBUG_PRINT("info",("log_space_total: %llu", uint64(rli->log_space_total)));
+  DBUG_PRINT("info",("log_space_total: %llu",rli->log_space_total));
   mysql_mutex_unlock(&rli->data_lock);
   DBUG_RETURN(error);
 }
@@ -1493,8 +1495,8 @@ Relay_log_info::alloc_inuse_relaylog(const char *name)
   rpl_gtid *gtid_list;
 
   gtid_count= relay_log_state.count();
-  if (!(gtid_list= (rpl_gtid *)my_malloc(sizeof(*gtid_list)*gtid_count,
-                                         MYF(MY_WME))))
+  if (!(gtid_list= (rpl_gtid *)my_malloc(PSI_INSTRUMENT_ME,
+                                 sizeof(*gtid_list)*gtid_count, MYF(MY_WME))))
   {
     my_error(ER_OUTOFMEMORY, MYF(0), (int)sizeof(*gtid_list)*gtid_count);
     return 1;
@@ -1642,8 +1644,8 @@ scan_one_gtid_slave_pos_table(THD *thd, HASH *hash, DYNAMIC_ARRAY *array,
     }
     else
     {
-      if (!(entry= (struct gtid_pos_element *)my_malloc(sizeof(*entry),
-                                                        MYF(MY_WME))))
+      if (!(entry= (struct gtid_pos_element *)my_malloc(PSI_INSTRUMENT_ME,
+                                                sizeof(*entry), MYF(MY_WME))))
       {
         my_error(ER_OUTOFMEMORY, MYF(0), (int)sizeof(*entry));
         err= 1;
@@ -1675,7 +1677,7 @@ end:
   {
     *out_hton= table->s->db_type();
     close_thread_tables(thd);
-    thd->mdl_context.release_transactional_locks(thd);
+    thd->mdl_context.release_transactional_locks();
   }
   return err;
 }
@@ -1702,7 +1704,7 @@ scan_all_gtid_slave_pos_table(THD *thd, int (*cb)(THD *, LEX_CSTRING *, void *),
   {
     my_error(ER_FILE_NOT_FOUND, MYF(0), path, my_errno);
     close_thread_tables(thd);
-    thd->release_transactional_locks();
+    thd->mdl_context.release_transactional_locks();
     return 1;
   }
   else
@@ -1715,7 +1717,7 @@ scan_all_gtid_slave_pos_table(THD *thd, int (*cb)(THD *, LEX_CSTRING *, void *),
     err= ha_discover_table_names(thd, &MYSQL_SCHEMA_NAME, dirp, &tl, false);
     my_dirend(dirp);
     close_thread_tables(thd);
-    thd->release_transactional_locks();
+    thd->mdl_context.release_transactional_locks();
     if (err)
       return err;
 
@@ -1884,10 +1886,11 @@ rpl_load_gtid_slave_state(THD *thd)
 
   cb_data.table_list= NULL;
   cb_data.default_entry= NULL;
-  my_hash_init(&hash, &my_charset_bin, 32,
+  my_hash_init(PSI_INSTRUMENT_ME, &hash, &my_charset_bin, 32,
                offsetof(gtid_pos_element, gtid) + offsetof(rpl_gtid, domain_id),
                sizeof(uint32), NULL, my_free, HASH_UNIQUE);
-  if ((err= my_init_dynamic_array(&array, sizeof(gtid_pos_element), 0, 0, MYF(0))))
+  if ((err= my_init_dynamic_array(PSI_INSTRUMENT_ME, &array,
+                                  sizeof(gtid_pos_element), 0, 0, MYF(0))))
     goto end;
   array_inited= true;
 
@@ -2000,7 +2003,7 @@ end:
     ha_commit_trans(thd, FALSE);
     ha_commit_trans(thd, TRUE);
     close_thread_tables(thd);
-    thd->release_transactional_locks();
+    thd->mdl_context.release_transactional_locks();
   }
 
   return err;
@@ -2282,7 +2285,14 @@ void rpl_group_info::cleanup_context(THD *thd, bool error)
 
   if (unlikely(error))
   {
-    thd->release_transactional_locks();
+    /*
+      trans_rollback above does not rollback XA transactions
+      (todo/fixme consider to do so.
+    */
+    if (thd->transaction->xid_state.is_explicit_XA())
+      xa_trans_force_rollback(thd);
+
+    thd->mdl_context.release_transactional_locks();
 
     if (thd == rli->sql_driver_thd)
     {
@@ -2396,10 +2406,10 @@ void rpl_group_info::slave_close_thread_tables(THD *thd)
   if (thd->transaction_rollback_request)
   {
     trans_rollback_implicit(thd);
-    thd->release_transactional_locks();
+    thd->mdl_context.release_transactional_locks();
   }
   else if (! thd->in_multi_stmt_transaction_mode())
-    thd->release_transactional_locks();
+    thd->mdl_context.release_transactional_locks();
   else
     thd->mdl_context.release_statement_locks();
 
