@@ -1,4 +1,4 @@
-/* Copyright (C) 2015-2021 Codership Oy <info@codership.com>
+/* Copyright (C) 2015-2019 Codership Oy <info@codership.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,7 +30,6 @@
 #include "wsrep_high_priority_service.h"
 #include "wsrep_storage_service.h"
 #include "wsrep_thd.h"
-#include "wsrep_server_state.h"
 
 #include <string>
 #include <sstream>
@@ -55,7 +54,7 @@ static const std::string create_cluster_table_str=
   "view_seqno BIGINT NOT NULL,"
   "protocol_version INT NOT NULL,"
   "capabilities INT NOT NULL"
-  ") ENGINE=InnoDB STATS_PERSISTENT=0";
+  ") ENGINE=InnoDB";
 
 static const std::string create_members_table_str=
   "CREATE TABLE IF NOT EXISTS " + wsrep_schema_str + "." + members_table_str +
@@ -64,7 +63,7 @@ static const std::string create_members_table_str=
   "cluster_uuid CHAR(36) NOT NULL,"
   "node_name CHAR(32) NOT NULL,"
   "node_incoming_address VARCHAR(256) NOT NULL"
-  ") ENGINE=InnoDB STATS_PERSISTENT=0";
+  ") ENGINE=InnoDB";
 
 #ifdef WSREP_SCHEMA_MEMBERS_HISTORY
 static const std::string cluster_member_history_table_str= "wsrep_cluster_member_history";
@@ -77,7 +76,7 @@ static const std::string create_members_history_table_str=
   "last_view_seqno BIGINT NOT NULL,"
   "node_name CHAR(32) NOT NULL,"
   "node_incoming_address VARCHAR(256) NOT NULL"
-  ") ENGINE=InnoDB STATS_PERSISTENT=0";
+  ") ENGINE=InnoDB";
 #endif /* WSREP_SCHEMA_MEMBERS_HISTORY */
 
 static const std::string create_frag_table_str=
@@ -89,7 +88,7 @@ static const std::string create_frag_table_str=
   "flags INT NOT NULL, "
   "frag LONGBLOB NOT NULL, "
   "PRIMARY KEY (node_uuid, trx_id, seqno)"
-  ") ENGINE=InnoDB STATS_PERSISTENT=0";
+  ") ENGINE=InnoDB";
 
 static const std::string delete_from_cluster_table=
   "DELETE FROM " + wsrep_schema_str + "." + cluster_table_str;
@@ -97,26 +96,6 @@ static const std::string delete_from_cluster_table=
 static const std::string delete_from_members_table=
   "DELETE FROM " + wsrep_schema_str + "." + members_table_str;
 
-/* For rolling upgrade we need to use ALTER. We do not want
-persistent statistics to be collected from these tables. */
-static const std::string alter_cluster_table=
-  "ALTER TABLE " + wsrep_schema_str + "." + cluster_table_str +
-  " STATS_PERSISTENT=0";
-
-static const std::string alter_members_table=
-  "ALTER TABLE " + wsrep_schema_str + "." + members_table_str +
-  " STATS_PERSISTENT=0";
-
-#ifdef WSREP_SCHEMA_MEMBERS_HISTORY
-static const std::string alter_members_history_table=
-  "ALTER TABLE " + wsrep_schema_str + "." + members_history_table_str +
-  " STATS_PERSISTENT=0";
-#endif
-
-static const std::string alter_frag_table=
-  "ALTER TABLE " + wsrep_schema_str + "." + sr_table_str +
-  " STATS_PERSISTENT=0";
-
 namespace Wsrep_schema_impl
 {
 
@@ -160,6 +139,25 @@ private:
   my_bool m_wsrep_on;
 };
 
+class thd_server_status
+{
+public:
+  thd_server_status(THD* thd, uint server_status, bool condition)
+    : m_thd(thd)
+    , m_thd_server_status(thd->server_status)
+  {
+    if (condition)
+      thd->server_status= server_status;
+  }
+  ~thd_server_status()
+  {
+    m_thd->server_status= m_thd_server_status;
+  }
+private:
+  THD* m_thd;
+  uint m_thd_server_status;
+};
+
 class thd_context_switch
 {
 public:
@@ -180,24 +178,6 @@ private:
   THD *m_cur_thd;
 };
 
-class sql_safe_updates
-{
-public:
-  sql_safe_updates(THD* thd)
-    : m_thd(thd)
-    , m_option_bits(thd->variables.option_bits)
-  {
-    thd->variables.option_bits&= ~OPTION_SAFE_UPDATES;
-  }
-  ~sql_safe_updates()
-  {
-    m_thd->variables.option_bits= m_option_bits;
-  }
-private:
-  THD* m_thd;
-  ulonglong m_option_bits;
-};
-
 static int execute_SQL(THD* thd, const char* sql, uint length) {
   DBUG_ENTER("Wsrep_schema::execute_SQL()");
   int err= 0;
@@ -269,10 +249,10 @@ static int open_table(THD* thd,
   tables.init_one_table(schema_name,
                         table_name,
                         NULL, lock_type);
-  thd->lex->query_tables_own_last= 0;
 
   if (!open_n_lock_single_table(thd, &tables, tables.lock_type, flags)) {
     close_thread_tables(thd);
+    my_error(ER_NO_SUCH_TABLE, MYF(0), schema_name->str, table_name->str);
     DBUG_RETURN(1);
   }
 
@@ -288,15 +268,8 @@ static int open_for_write(THD* thd, const char* table_name, TABLE** table) {
   LEX_CSTRING table_str= { table_name, strlen(table_name) };
   if (Wsrep_schema_impl::open_table(thd, &schema_str, &table_str, TL_WRITE,
                                     table)) {
-    // No need to log an error if the query was bf aborted,
-    // thd client will get ER_LOCK_DEADLOCK in the end.
-    const bool interrupted= thd->killed ||
-      (thd->is_error() &&
-       (thd->get_stmt_da()->sql_errno() == ER_QUERY_INTERRUPTED));
-    if (!interrupted) {
-      WSREP_ERROR("Failed to open table %s.%s for writing",
-                  schema_str.str, table_name);
-    }
+    WSREP_ERROR("Failed to open table %s.%s for writing",
+                schema_str.str, table_name);
     return 1;
   }
   empty_record(*table);
@@ -519,12 +492,11 @@ static int scan(TABLE* table, uint field, INTTYPE& val)
 
 static int scan(TABLE* table, uint field, char* strbuf, uint strbuf_len)
 {
-  String str;
-  (void)table->field[field]->val_str(&str);
-  LEX_CSTRING tmp= str.lex_cstring();
-  uint len = tmp.length;
-  strncpy(strbuf, tmp.str, std::min(len, strbuf_len));
-  strbuf[strbuf_len - 1]= '\0';
+  uint len;
+  StringBuffer<STRING_BUFFER_USUAL_SIZE> str;
+  (void) table->field[field]->val_str(&str);
+  len= str.length();
+  strmake(strbuf, str.ptr(), MY_MIN(len, strbuf_len-1));
   return 0;
 }
 
@@ -599,33 +571,21 @@ static int init_for_index_scan(TABLE* table, const uchar* key,
  */
 static int end_index_scan(TABLE* table) {
   int error;
-  if (table->file->inited) {
-    if ((error= table->file->ha_index_end())) {
-      WSREP_ERROR("Failed to end scan: %d", error);
-      return 1;
-    }
+  if ((error= table->file->ha_index_end())) {
+    WSREP_ERROR("Failed to end scan: %d", error);
+    return 1;
   }
   return 0;
 }
 
-static void make_key(TABLE* table, uchar** key, key_part_map* map, int parts) {
+static void make_key(TABLE* table, uchar* key, key_part_map* map, int parts) {
   uint prefix_length= 0;
   KEY_PART_INFO* key_part= table->key_info->key_part;
-
   for (int i=0; i < parts; i++)
     prefix_length += key_part[i].store_length;
-
   *map= make_prev_keypart_map(parts);
-
-  if (!(*key= (uchar *) my_malloc(prefix_length + 1, MYF(MY_WME))))
-  {
-    WSREP_ERROR("Failed to allocate memory for key prefix_length %u", prefix_length);
-    assert(0);
-  }
-
-  key_copy(*key, table->record[0], table->key_info, prefix_length);
+  key_copy(key, table->record[0], table->key_info, prefix_length);
 }
-
 } /* namespace Wsrep_schema_impl */
 
 
@@ -645,15 +605,13 @@ static void wsrep_init_thd_for_schema(THD *thd)
 
   thd->prior_thr_create_utime= thd->start_utime= thd->thr_create_utime;
 
-  /* No Galera replication */
-  thd->variables.wsrep_on= 0;
+  /* */
+  thd->variables.wsrep_on    = 0;
   /* No binlogging */
-  thd->variables.sql_log_bin= 0;
-  thd->variables.option_bits&= ~OPTION_BIN_LOG;
-  /* No safe updates */
-  thd->variables.option_bits&= ~OPTION_SAFE_UPDATES;
+  thd->variables.sql_log_bin = 0;
+  thd->variables.option_bits &= ~OPTION_BIN_LOG;
   /* No general log */
-  thd->variables.option_bits|= OPTION_LOG_OFF;
+  thd->variables.option_bits |= OPTION_LOG_OFF;
   /* Read committed isolation to avoid gap locking */
   thd->variables.tx_isolation= ISO_READ_COMMITTED;
   wsrep_assign_from_threadvars(thd);
@@ -680,27 +638,13 @@ int Wsrep_schema::init()
       Wsrep_schema_impl::execute_SQL(thd,
                                      create_members_history_table_str.c_str(),
                                      create_members_history_table_str.size()) ||
-      Wsrep_schema_impl::execute_SQL(thd,
-                                     alter_members_history_table.c_str(),
-                                     alter_members_history_table.size()) ||
 #endif /* WSREP_SCHEMA_MEMBERS_HISTORY */
       Wsrep_schema_impl::execute_SQL(thd,
                                      create_frag_table_str.c_str(),
-                                     create_frag_table_str.size()) ||
-      Wsrep_schema_impl::execute_SQL(thd,
-                                     alter_cluster_table.c_str(),
-                                     alter_cluster_table.size()) ||
-      Wsrep_schema_impl::execute_SQL(thd,
-                                     alter_members_table.c_str(),
-                                     alter_members_table.size()) ||
-      Wsrep_schema_impl::execute_SQL(thd,
-                                     alter_frag_table.c_str(),
-	                             alter_frag_table.size()))
-  {
+                                     create_frag_table_str.size())) {
     ret= 1;
   }
-  else
-  {
+  else {
     ret= 0;
   }
 
@@ -722,7 +666,6 @@ int Wsrep_schema::store_view(THD* thd, const Wsrep_view& view)
 
   Wsrep_schema_impl::wsrep_off wsrep_off(thd);
   Wsrep_schema_impl::binlog_off binlog_off(thd);
-  Wsrep_schema_impl::sql_safe_updates sql_safe_updates(thd);
 
   /*
     Clean up cluster table and members table.
@@ -925,7 +868,7 @@ Wsrep_view Wsrep_schema::restore_view(THD* thd, const Wsrep_id& own_id) const {
       close_thread_tables(thd);
     }
   }
-  thd->release_transactional_locks();
+  thd->mdl_context.release_transactional_locks();
 
   thd->variables.wsrep_sync_wait= wsrep_sync_wait_saved;
 
@@ -969,22 +912,13 @@ int Wsrep_schema::append_fragment(THD* thd,
               thd->thread_id,
               os.str().c_str(),
               transaction_id.get());
-  /* use private query table list for the duration of fragment storing,
-     populated query table list from "parent DML" may cause problems .e.g
-     for virtual column handling
- */
-  Query_tables_list query_tables_list_backup;
-  thd->lex->reset_n_backup_query_tables_list(&query_tables_list_backup);
-
   Wsrep_schema_impl::binlog_off binlog_off(thd);
-  Wsrep_schema_impl::sql_safe_updates sql_safe_updates(thd);
   Wsrep_schema_impl::init_stmt(thd);
 
   TABLE* frag_table= 0;
   if (Wsrep_schema_impl::open_for_write(thd, sr_table_str.c_str(), &frag_table))
   {
     trans_rollback_stmt(thd);
-    thd->lex->restore_backup_query_tables_list(&query_tables_list_backup);
     DBUG_RETURN(1);
   }
 
@@ -998,11 +932,9 @@ int Wsrep_schema::append_fragment(THD* thd,
   if ((error= Wsrep_schema_impl::insert(frag_table))) {
     WSREP_ERROR("Failed to write to frag table: %d", error);
     trans_rollback_stmt(thd);
-    thd->lex->restore_backup_query_tables_list(&query_tables_list_backup);
     DBUG_RETURN(1);
   }
   Wsrep_schema_impl::finish_stmt(thd);
-  thd->lex->restore_backup_query_tables_list(&query_tables_list_backup);
   DBUG_RETURN(0);
 }
 
@@ -1019,24 +951,15 @@ int Wsrep_schema::update_fragment_meta(THD* thd,
               ws_meta.seqno().get());
   DBUG_ASSERT(ws_meta.seqno().is_undefined() == false);
 
-  /* use private query table list for the duration of fragment storing,
-     populated query table list from "parent DML" may cause problems .e.g
-     for virtual column handling
- */
-  Query_tables_list query_tables_list_backup;
-  thd->lex->reset_n_backup_query_tables_list(&query_tables_list_backup);
-
   Wsrep_schema_impl::binlog_off binlog_off(thd);
-  Wsrep_schema_impl::sql_safe_updates sql_safe_updates(thd);
   int error;
-  uchar *key=NULL;
+  uchar key[MAX_KEY_LENGTH];
   key_part_map key_map= 0;
   TABLE* frag_table= 0;
 
   Wsrep_schema_impl::init_stmt(thd);
   if (Wsrep_schema_impl::open_for_write(thd, sr_table_str.c_str(), &frag_table))
   {
-    thd->lex->restore_backup_query_tables_list(&query_tables_list_backup);
     DBUG_RETURN(1);
   }
 
@@ -1044,7 +967,7 @@ int Wsrep_schema::update_fragment_meta(THD* thd,
   Wsrep_schema_impl::store(frag_table, 0, ws_meta.server_id());
   Wsrep_schema_impl::store(frag_table, 1, ws_meta.transaction_id().get());
   Wsrep_schema_impl::store(frag_table, 2, -1);
-  Wsrep_schema_impl::make_key(frag_table, &key, &key_map, 3);
+  Wsrep_schema_impl::make_key(frag_table, key, &key_map, 3);
 
   if ((error= Wsrep_schema_impl::init_for_index_scan(frag_table,
                                                      key, key_map)))
@@ -1057,12 +980,9 @@ int Wsrep_schema::update_fragment_meta(THD* thd,
                  error);
     }
     Wsrep_schema_impl::finish_stmt(thd);
-    thd->lex->restore_backup_query_tables_list(&query_tables_list_backup);
-    my_free(key);
     DBUG_RETURN(1);
   }
 
-  my_free(key);
   /* Copy the original record to frag_table->record[1] */
   store_record(frag_table, record[1]);
 
@@ -1075,13 +995,11 @@ int Wsrep_schema::update_fragment_meta(THD* thd,
                 frag_table->s->table_name.str,
                 error);
     Wsrep_schema_impl::finish_stmt(thd);
-    thd->lex->restore_backup_query_tables_list(&query_tables_list_backup);
     DBUG_RETURN(1);
   }
 
   int ret= Wsrep_schema_impl::end_index_scan(frag_table);
   Wsrep_schema_impl::finish_stmt(thd);
-  thd->lex->restore_backup_query_tables_list(&query_tables_list_backup);
   DBUG_RETURN(ret);
 }
 
@@ -1097,7 +1015,7 @@ static int remove_fragment(THD*                  thd,
               seqno.get());
   int ret= 0;
   int error;
-  uchar *key= NULL;
+  uchar key[MAX_KEY_LENGTH];
   key_part_map key_map= 0;
 
   DBUG_ASSERT(server_id.is_undefined() == false);
@@ -1111,7 +1029,7 @@ static int remove_fragment(THD*                  thd,
   Wsrep_schema_impl::store(frag_table, 0, server_id);
   Wsrep_schema_impl::store(frag_table, 1, transaction_id.get());
   Wsrep_schema_impl::store(frag_table, 2, seqno.get());
-  Wsrep_schema_impl::make_key(frag_table, &key, &key_map, 3);
+  Wsrep_schema_impl::make_key(frag_table, key, &key_map, 3);
 
   if ((error= Wsrep_schema_impl::init_for_index_scan(frag_table,
                                                      key,
@@ -1133,8 +1051,6 @@ static int remove_fragment(THD*                  thd,
     ret= 1;
   }
 
-  if (key)
-    my_free(key);
   Wsrep_schema_impl::end_index_scan(frag_table);
   return ret;
 }
@@ -1150,7 +1066,6 @@ int Wsrep_schema::remove_fragments(THD* thd,
   WSREP_DEBUG("Removing %zu fragments", fragments.size());
   Wsrep_schema_impl::wsrep_off  wsrep_off(thd);
   Wsrep_schema_impl::binlog_off binlog_off(thd);
-  Wsrep_schema_impl::sql_safe_updates sql_safe_updates(thd);
 
   Query_tables_list query_tables_list_backup;
   Open_tables_backup open_tables_backup;
@@ -1197,6 +1112,9 @@ int Wsrep_schema::remove_fragments(THD* thd,
   }
   else
   {
+    Wsrep_schema_impl::thd_server_status
+      thd_server_status(thd, thd->server_status | SERVER_STATUS_IN_TRANS,
+                        thd->in_multi_stmt_transaction_mode());
     Wsrep_schema_impl::finish_stmt(thd);
   }
 
@@ -1218,13 +1136,12 @@ int Wsrep_schema::replay_transaction(THD* orig_thd,
 
   Wsrep_schema_impl::wsrep_off  wsrep_off(&thd);
   Wsrep_schema_impl::binlog_off binlog_off(&thd);
-  Wsrep_schema_impl::sql_safe_updates sql_safe_updates(&thd);
   Wsrep_schema_impl::thd_context_switch thd_context_switch(orig_thd, &thd);
 
   int ret= 1;
   int error;
   TABLE* frag_table= 0;
-  uchar *key=NULL;
+  uchar key[MAX_KEY_LENGTH];
   key_part_map key_map= 0;
 
   for (std::vector<wsrep::seqno>::const_iterator i= fragments.begin();
@@ -1241,7 +1158,7 @@ int Wsrep_schema::replay_transaction(THD* orig_thd,
     Wsrep_schema_impl::store(frag_table, 0, ws_meta.server_id());
     Wsrep_schema_impl::store(frag_table, 1, ws_meta.transaction_id().get());
     Wsrep_schema_impl::store(frag_table, 2, i->get());
-    Wsrep_schema_impl::make_key(frag_table, &key, &key_map, 3);
+    Wsrep_schema_impl::make_key(frag_table, key, &key_map, 3);
 
     int error= Wsrep_schema_impl::init_for_index_scan(frag_table,
                                                       key,
@@ -1288,7 +1205,6 @@ int Wsrep_schema::replay_transaction(THD* orig_thd,
       Wsrep_schema_impl::finish_stmt(&thd);
       DBUG_RETURN(1);
     }
-
     error= Wsrep_schema_impl::init_for_index_scan(frag_table,
                                                   key,
                                                   key_map);
@@ -1302,7 +1218,6 @@ int Wsrep_schema::replay_transaction(THD* orig_thd,
     }
 
     error= Wsrep_schema_impl::delete_row(frag_table);
-
     if (error)
     {
       WSREP_WARN("Could not delete row from streaming log table: %d", error);
@@ -1312,12 +1227,8 @@ int Wsrep_schema::replay_transaction(THD* orig_thd,
     }
     Wsrep_schema_impl::end_index_scan(frag_table);
     Wsrep_schema_impl::finish_stmt(&thd);
-    my_free(key);
-    key= NULL;
   }
 
-  if (key)
-    my_free(key);
   DBUG_RETURN(ret);
 }
 
@@ -1333,7 +1244,6 @@ int Wsrep_schema::recover_sr_transactions(THD *orig_thd)
   Wsrep_storage_service storage_service(&storage_thd);
   Wsrep_schema_impl::binlog_off binlog_off(&storage_thd);
   Wsrep_schema_impl::wsrep_off wsrep_off(&storage_thd);
-  Wsrep_schema_impl::sql_safe_updates sql_safe_updates(&storage_thd);
   Wsrep_schema_impl::thd_context_switch thd_context_switch(orig_thd,
                                                            &storage_thd);
   Wsrep_server_state& server_state(Wsrep_server_state::instance());
@@ -1392,7 +1302,7 @@ int Wsrep_schema::recover_sr_transactions(THD *orig_thd)
     goto out;
   }
 
-  while (true)
+  while (0 == error)
   {
     if ((error= Wsrep_schema_impl::next_record(frag_table)) == 0)
     {
@@ -1442,19 +1352,23 @@ int Wsrep_schema::recover_sr_transactions(THD *orig_thd)
       }
       applier->store_globals();
       wsrep::mutable_buffer unused;
-      applier->apply_write_set(ws_meta, data, unused);
-      applier->after_apply();
+      if ((ret= applier->apply_write_set(ws_meta, data, unused)) != 0)
+      {
+        WSREP_ERROR("SR trx recovery applying returned %d", ret);
+      }
+      else
+      {
+        applier->after_apply();
+      }
       storage_service.store_globals();
     }
     else if (error == HA_ERR_END_OF_FILE)
     {
       ret= 0;
-      break;
     }
     else
     {
       WSREP_ERROR("SR table scan returned error %d", error);
-      break;
     }
   }
   Wsrep_schema_impl::end_scan(frag_table);
