@@ -1,4 +1,4 @@
-/* Copyright 2008-2020 Codership Oy <http://www.codership.com>
+/* Copyright 2008-2017 Codership Oy <http://www.codership.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -245,7 +245,7 @@ static bool sst_auth_real_set (const char* value)
 
   if (value)
   {
-    v= my_strdup(value, MYF(0));
+    v= my_strdup(PSI_INSTRUMENT_ME, value, MYF(0));
   }
   else                                          // its NULL
   {
@@ -263,14 +263,8 @@ static bool sst_auth_real_set (const char* value)
     if (strlen(sst_auth_real))
     {
       if (wsrep_sst_auth) { my_free((void*) wsrep_sst_auth); }
-      wsrep_sst_auth= my_strdup(WSREP_SST_AUTH_MASK, MYF(0));
+      wsrep_sst_auth= my_strdup(PSI_INSTRUMENT_ME, WSREP_SST_AUTH_MASK, MYF(0));
     }
-    else
-    {
-      if (wsrep_sst_auth) { my_free((void*) wsrep_sst_auth); }
-      wsrep_sst_auth= NULL;
-    }
-
     return 0;
   }
   return 1;
@@ -314,40 +308,12 @@ bool wsrep_before_SE()
 }
 
 // Signal end of SST
-static bool wsrep_sst_complete (THD*                thd,
-                                int const           rcode,
-                                wsrep::gtid const   sst_gtid)
+static void wsrep_sst_complete (THD*                thd,
+                                int const           rcode)
 {
   Wsrep_client_service client_service(thd, thd->wsrep_cs());
-  Wsrep_server_state& server_state= Wsrep_server_state::instance();
-  enum wsrep::server_state::state state= server_state.state();
-  bool failed= false;
-  char start_pos_buf[FN_REFLEN];
-  ssize_t len= wsrep::print_to_c_str(sst_gtid, start_pos_buf, FN_REFLEN-1);
-  start_pos_buf[len]='\0';
-
-  // Do not call sst_received if we are not in joiner or
-  // initialized state on server. This is because it
-  // assumes we are on those states. Give error if we are
-  // in incorrect state.
-  if ((state == Wsrep_server_state::s_joiner ||
-       state == Wsrep_server_state::s_initialized))
-  {
-    Wsrep_server_state::instance().sst_received(client_service,
-       rcode);
-    WSREP_INFO("SST succeeded for position %s", start_pos_buf);
-  }
-  else
-  {
-    WSREP_ERROR("SST failed for position %s initialized %d server_state %s",
-                start_pos_buf,
-                server_state.is_initialized(),
-                wsrep::to_c_string(state));
-    failed= true;
-  }
-
+  Wsrep_server_state::instance().sst_received(client_service, rcode);
   wsrep_joiner_monitor_end();
-  return failed;
 }
 
   /*
@@ -359,15 +325,13 @@ static bool wsrep_sst_complete (THD*                thd,
   @param seqno     [IN]               Initial state sequence number
   @param state     [IN]               Always NULL, also ignored by wsrep provider (?)
   @param state_len [IN]               Always 0, also ignored by wsrep provider (?)
-  @return true when successful, false if error
 */
-bool wsrep_sst_received (THD*                thd,
+void wsrep_sst_received (THD*                thd,
                          const wsrep_uuid_t& uuid,
                          wsrep_seqno_t const seqno,
                          const void* const   state,
                          size_t const        state_len)
 {
-  bool error= false;
   /*
     To keep track of whether the local uuid:seqno should be updated. Also, note
     that local state (uuid:seqno) is updated/checkpointed only after we get an
@@ -387,8 +351,8 @@ bool wsrep_sst_received (THD*                thd,
                                wsrep::seqno(seqno));
 
     if (!wsrep_before_SE()) {
-      wsrep_set_SE_checkpoint(wsrep::gtid::undefined());
-      wsrep_set_SE_checkpoint(sst_gtid);
+      wsrep_set_SE_checkpoint(wsrep::gtid::undefined(), wsrep_gtid_server.undefined());
+      wsrep_set_SE_checkpoint(sst_gtid, wsrep_gtid_server.gtid());
     }
     wsrep_verify_SE_checkpoint(uuid, seqno);
 
@@ -400,17 +364,15 @@ bool wsrep_sst_received (THD*                thd,
       wsrep_store_threadvars(thd);
     }
     else {
-      my_pthread_setspecific_ptr(THR_THD, NULL);
+      set_current_thd(nullptr);
     }
 
     /* During sst WSREP(thd) is not yet set for joiner. */
     if (WSREP_ON)
     {
       int const rcode(seqno < 0 ? seqno : 0);
-      error= wsrep_sst_complete(thd,rcode, sst_gtid);
+      wsrep_sst_complete(thd,rcode);
     }
-
-    return error;
 }
 
 static int sst_scan_uuid_seqno (const char* str,
@@ -453,7 +415,7 @@ static char* generate_name_value(const char* name, const char* value)
   size_t name_len= strlen(name);
   size_t value_len= strlen(value);
   char* buf=
-    (char*) my_malloc((name_len + value_len + 5) * sizeof(char), MYF(0));
+    (char*) my_malloc(PSI_INSTRUMENT_ME, (name_len + value_len + 5), MYF(0));
   if (buf)
   {
     char* ref= buf;
@@ -488,11 +450,11 @@ static int generate_binlog_opt_val(char** ret)
     *ret= strcmp(opt_bin_logname, "0") ?
       generate_name_value(WSREP_SST_OPT_BINLOG,
                           opt_bin_logname) :
-      my_strdup("", MYF(0));
+      my_strdup(PSI_INSTRUMENT_ME, "", MYF(0));
   }
   else
   {
-    *ret= my_strdup("", MYF(0));
+    *ret= my_strdup(PSI_INSTRUMENT_ME, "", MYF(0));
   }
   if (!*ret) return -ENOMEM;
   return 0;
@@ -507,11 +469,11 @@ static int generate_binlog_index_opt_val(char** ret)
     *ret= strcmp(opt_binlog_index_name, "0") ?
       generate_name_value(WSREP_SST_OPT_BINLOG_INDEX,
                           opt_binlog_index_name) :
-      my_strdup("", MYF(0));
+      my_strdup(PSI_INSTRUMENT_ME, "", MYF(0));
   }
   else
   {
-    *ret= my_strdup("", MYF(0));
+    *ret= my_strdup(PSI_INSTRUMENT_ME, "", MYF(0));
   }
   if (!*ret) return -ENOMEM;
   return 0;
@@ -633,7 +595,7 @@ static void* sst_joiner_thread (void* a)
             err= EINVAL;
             goto err;
           } else {
-            wsrep_gtid_domain_id= (uint32) domain_id;
+            wsrep_gtid_server.domain_id= (uint32) domain_id;
           }
         }
       }
@@ -691,7 +653,7 @@ err:
     /* Read committed isolation to avoid gap locking */
     thd->variables.tx_isolation= ISO_READ_COMMITTED;
 
-    wsrep_sst_complete (thd, -err, ret_gtid);
+    wsrep_sst_complete (thd, -err);
 
     delete thd;
     my_thread_end();
@@ -700,75 +662,51 @@ err:
   return NULL;
 }
 
-#define WSREP_SST_AUTH_ENV        "WSREP_SST_OPT_AUTH"
-#define WSREP_SST_REMOTE_AUTH_ENV "WSREP_SST_OPT_REMOTE_AUTH"
-#define DATA_HOME_DIR_ENV         "INNODB_DATA_HOME_DIR"
+#define WSREP_SST_AUTH_ENV "WSREP_SST_OPT_AUTH"
 
-static int sst_append_env_var(wsp::env&   env,
-                              const char* const var,
-                              const char* const val)
+static int sst_append_auth_env(wsp::env& env, const char* sst_auth)
 {
-  int const env_str_size= strlen(var) + 1 /* = */
-                          + (val ? strlen(val) : 0) + 1 /* \0 */;
+  int const sst_auth_size= strlen(WSREP_SST_AUTH_ENV) + 1 /* = */
+    + (sst_auth ? strlen(sst_auth) : 0) + 1 /* \0 */;
 
-  wsp::string env_str(env_str_size); // for automatic cleanup on return
-  if (!env_str()) return -ENOMEM;
+  wsp::string sst_auth_str(sst_auth_size); // for automatic cleanup on return
+  if (!sst_auth_str()) return -ENOMEM;
 
-  int ret= snprintf(env_str(), env_str_size, "%s=%s", var, val ? val : "");
+  int ret= snprintf(sst_auth_str(), sst_auth_size, "%s=%s",
+                    WSREP_SST_AUTH_ENV, sst_auth ? sst_auth : "");
 
-  if (ret < 0 || ret >= env_str_size)
+  if (ret < 0 || ret >= sst_auth_size)
   {
-    WSREP_ERROR("sst_append_env_var(): snprintf(%s=%s) failed: %d",
-                var, val, ret);
+    WSREP_ERROR("sst_append_auth_env(): snprintf() failed: %d", ret);
     return (ret < 0 ? ret : -EMSGSIZE);
   }
 
-  env.append(env_str());
+  env.append(sst_auth_str());
   return -env.error();
 }
 
-#ifdef __WIN__
-/*
-  Space, single quote, ampersand, backquote, I/O redirection
-  characters, caret, all brackets, plus, exclamation and comma
-  characters require text to be enclosed in double quotes:
-*/
-#define IS_SPECIAL(c) \
-  (isspace(c) || c == '\'' || c == '&' || c == '`' || c == '|' || \
-                 c ==  '>' || c == '<' || c == ';' || c == '^' || \
-                 c ==  '[' || c == ']' || c == '{' || c == '}' || \
-                 c ==  '(' || c == ')' || c == '+' || c == '!' || \
-                 c ==  ',')
-/*
-  Inside values, equals character are interpreted as special
-  character and requires quotation:
-*/
-#define IS_SPECIAL_V(c) (IS_SPECIAL(c) || c == '=')
-/*
-  Double quotation mark and percent characters require escaping:
-*/
-#define IS_REQ_ESCAPING(c) (c == '""' || c == '%')
-#else
-/*
-  Space, single quote, ampersand, backquote, and I/O redirection
-  characters require text to be enclosed in double quotes. The
-  semicolon is used to separate shell commands, so it must be
-  enclosed in double quotes as well:
-*/
-#define IS_SPECIAL(c) \
-  (isspace(c) || c == '\'' || c == '&' || c == '`' || c == '|' || \
-                 c ==  '>' || c == '<' || c == ';')
-/*
-  Inside values, characters are interpreted as in parameter names:
-*/
-#define IS_SPECIAL_V(c) IS_SPECIAL(c)
-/*
-  Double quotation mark and backslash characters require
-  backslash prefixing, the dollar symbol is used to substitute
-  a variable value, therefore it also requires escaping:
-*/
-#define IS_REQ_ESCAPING(c) (c == '"' || c == '\\' || c == '$')
-#endif
+#define DATA_HOME_DIR_ENV "INNODB_DATA_HOME_DIR"
+
+static int sst_append_data_dir(wsp::env& env, const char* data_dir)
+{
+  int const data_dir_size= strlen(DATA_HOME_DIR_ENV) + 1 /* = */
+    + (data_dir ? strlen(data_dir) : 0) + 1 /* \0 */;
+
+  wsp::string data_dir_str(data_dir_size); // for automatic cleanup on return
+  if (!data_dir_str()) return -ENOMEM;
+
+  int ret= snprintf(data_dir_str(), data_dir_size, "%s=%s",
+                    DATA_HOME_DIR_ENV, data_dir ? data_dir : "");
+
+  if (ret < 0 || ret >= data_dir_size)
+  {
+    WSREP_ERROR("sst_append_data_dir(): snprintf() failed: %d", ret);
+    return (ret < 0 ? ret : -EMSGSIZE);
+  }
+
+  env.append(data_dir_str());
+  return -env.error();
+}
 
 static size_t estimate_cmd_len (bool* extra_args)
 {
@@ -794,17 +732,11 @@ static size_t estimate_cmd_len (bool* extra_args)
       char c;
       while ((c = *arg++) != 0)
       {
-        if (IS_SPECIAL(c))
+        /* A whitespace or a single quote requires double quotation marks: */
+        if (isspace(c) || c == '\'')
         {
           quotation= true;
         }
-        else if (IS_REQ_ESCAPING(c))
-        {
-          cmd_len++;
-#ifdef __WIN__
-          quotation= true;
-#endif
-        }
         /*
           If the equals symbol is encountered, then we need to separately
           process the right side:
@@ -823,20 +755,49 @@ static size_t estimate_cmd_len (bool* extra_args)
           }
           while ((c = *arg++) != 0)
           {
-            if (IS_SPECIAL_V(c))
+            /*
+              A whitespace or a single quote requires double
+              quotation marks:
+            */
+            if (isspace(c) || c == '\'')
             {
               quotation= true;
             }
-            else if (IS_REQ_ESCAPING(c))
+            /*
+              Double quotation mark or backslash symbol requires backslash
+              prefixing:
+            */
+#ifdef __WIN__
+            else if (c == '"' || c == '\\')
+#else
+            /*
+              The dollar symbol is used to substitute a variable, therefore
+              it also requires escaping:
+            */
+            else if (c == '"' || c == '\\' || c == '$')
+#endif
             {
               cmd_len++;
-#ifdef __WIN__
-              quotation= true;
-#endif
             }
           }
           break;
         }
+        /*
+          Double quotation mark or backslash symbol requires backslash
+          prefixing:
+        */
+#ifdef __WIN__
+        else if (c == '"' || c == '\\')
+#else
+        /*
+          The dollar symbol is used to substitute a variable, therefore
+          it also requires escaping:
+        */
+        else if (c == '"' || c == '\\' || c == '$')
+#endif
+        {
+          cmd_len++;
+        }
       }
       /* Perhaps we need to quote the entire argument or its right part: */
       if (quotation)
@@ -879,17 +840,11 @@ static void copy_orig_argv (char* cmd_str)
       char c;
       while ((c = *arg_scan++) != 0)
       {
-        if (IS_SPECIAL(c))
+        /* A whitespace or a single quote requires double quotation marks: */
+        if (isspace(c) || c == '\'')
         {
           quotation= true;
         }
-        else if (IS_REQ_ESCAPING(c))
-        {
-          plain= false;
-#ifdef __WIN__
-          quotation= true;
-#endif
-        }
         /*
           If the equals symbol is encountered, then we need to separately
           process the right side:
@@ -924,13 +879,13 @@ static void copy_orig_argv (char* cmd_str)
               while (m)
               {
                 c = *arg++;
-                if (IS_REQ_ESCAPING(c))
-                {
 #ifdef __WIN__
-                  *cmd_str++ = c;
+                if (c == '"' || c == '\\')
 #else
-                  *cmd_str++ = '\\';
+                if (c == '"' || c == '\\' || c == '$')
 #endif
+                {
+                  *cmd_str++ = '\\';
                 }
                 *cmd_str++ = c;
                 m--;
@@ -959,20 +914,49 @@ static void copy_orig_argv (char* cmd_str)
           /* Let's deal with the left side of the expression: */
           while ((c = *arg_scan++) != 0)
           {
-            if (IS_SPECIAL_V(c))
+            /*
+              A whitespace or a single quote requires double
+              quotation marks:
+            */
+            if (isspace(c) || c == '\'')
             {
               quotation= true;
             }
-            else if (IS_REQ_ESCAPING(c))
+            /*
+              Double quotation mark or backslash symbol requires backslash
+              prefixing:
+            */
+#ifdef __WIN__
+            else if (c == '"' || c == '\\')
+#else
+            /*
+              The dollar symbol is used to substitute a variable, therefore
+              it also requires escaping:
+            */
+            else if (c == '"' || c == '\\' || c == '$')
+#endif
             {
               plain= false;
-#ifdef __WIN__
-              quotation= true;
-#endif
             }
           }
           break;
         }
+        /*
+          Double quotation mark or backslash symbol requires backslash
+          prefixing:
+        */
+#ifdef __WIN__
+        else if (c == '"' || c == '\\')
+#else
+        /*
+          The dollar symbol is used to substitute a variable, therefore
+          it also requires escaping:
+        */
+        else if (c == '"' || c == '\\' || c == '$')
+#endif
+        {
+          plain= false;
+        }
       }
       if (n)
       {
@@ -995,13 +979,13 @@ static void copy_orig_argv (char* cmd_str)
         {
           while ((c = *arg++) != 0)
           {
-            if (IS_REQ_ESCAPING(c))
-            {
 #ifdef __WIN__
-              *cmd_str++ = c;
+            if (c == '"' || c == '\\')
 #else
-              *cmd_str++ = '\\';
+            if (c == '"' || c == '\\' || c == '$')
 #endif
+            {
+              *cmd_str++ = '\\';
             }
             *cmd_str++ = c;
           }
@@ -1053,8 +1037,6 @@ static ssize_t sst_prepare_other (const char*  method,
   {
     WSREP_ERROR("sst_prepare_other(): generate_binlog_index_opt_val() failed %d",
                 ret);
-    if (binlog_opt_val) my_free(binlog_opt_val);
-    return ret;
   }
 
   make_wsrep_defaults_file();
@@ -1072,7 +1054,6 @@ static ssize_t sst_prepare_other (const char*  method,
                  wsrep_defaults_file,
                  (int)getpid(),
                  binlog_opt_val, binlog_index_opt_val);
-
   my_free(binlog_opt_val);
   my_free(binlog_index_opt_val);
 
@@ -1092,7 +1073,7 @@ static ssize_t sst_prepare_other (const char*  method,
     return -env.error();
   }
 
-  if ((ret= sst_append_env_var(env, WSREP_SST_AUTH_ENV, sst_auth)))
+  if ((ret= sst_append_auth_env(env, sst_auth)))
   {
     WSREP_ERROR("sst_prepare_other(): appending auth failed: %d", ret);
     return ret;
@@ -1100,7 +1081,7 @@ static ssize_t sst_prepare_other (const char*  method,
 
   if (data_home_dir)
   {
-    if ((ret= sst_append_env_var(env, DATA_HOME_DIR_ENV, data_home_dir)))
+    if ((ret= sst_append_data_dir(env, data_home_dir)))
     {
       WSREP_ERROR("sst_prepare_other(): appending data "
                   "directory failed: %d", ret);
@@ -1189,19 +1170,6 @@ static ssize_t sst_prepare_mysqldump (const char*  addr_in,
     *addr_out= addr_in;
   }
 
-  pthread_t monitor;
-  ret = mysql_thread_create (key_wsrep_sst_joiner_monitor, &monitor, NULL, wsrep_sst_joiner_monitor_thread, NULL);
-
-  if (ret)
-  {
-    WSREP_ERROR("sst_prepare_other(): mysql_thread_create() failed: %d (%s)",
-                ret, strerror(ret));
-    return -ret;
-  }
-
-  sst_joiner_completed= false;
-  pthread_detach (monitor);
-
   return ret;
 }
 
@@ -1312,7 +1280,7 @@ std::string wsrep_sst_prepare()
 
   const char* method_ptr(ret.data());
   const char* addr_ptr(ret.data() + strlen(method_ptr) + 1);
-  WSREP_DEBUG("Prepared SST request: %s|%s", method_ptr, addr_ptr);
+  WSREP_INFO ("Prepared SST request: %s|%s", method_ptr, addr_ptr);
 
   if (addr_out != addr_in) /* malloc'ed */ free ((char*)addr_out);
 
@@ -1398,13 +1366,15 @@ static int sst_donate_mysqldump (const char*         addr,
                      WSREP_SST_OPT_LPORT " '%u' "
                      WSREP_SST_OPT_SOCKET " '%s' "
                      "%s"
-                     WSREP_SST_OPT_GTID " '%s:%lld' "
+                     WSREP_SST_OPT_GTID " '%s:%lld,%d-%d-%llu' "
                      WSREP_SST_OPT_GTID_DOMAIN_ID " '%d'"
                      "%s",
                      addr, port, mysqld_port, mysqld_unix_port,
                      wsrep_defaults_file,
                      uuid_oss.str().c_str(), gtid.seqno().get(),
-                     wsrep_gtid_domain_id,
+                     wsrep_gtid_server.domain_id, wsrep_gtid_server.server_id,
+                     wsrep_gtid_server.seqno(),
+                     wsrep_gtid_server.domain_id,
                      bypass ? " " WSREP_SST_OPT_BYPASS : "");
 
   if (ret < 0 || size_t(ret) >= cmd_len)
@@ -1571,7 +1541,7 @@ static int sst_flush_tables(THD* thd)
     */
     char content[100];
     snprintf(content, sizeof(content), "%s:%lld %d\n", wsrep_cluster_state_uuid,
-             (long long)wsrep_locked_seqno, wsrep_gtid_domain_id);
+             (long long)wsrep_locked_seqno, wsrep_gtid_server.domain_id);
     err= sst_create_file(flush_success, content);
 
     const char base_name[]= "tables_flushed";
@@ -1596,7 +1566,7 @@ static int sst_flush_tables(THD* thd)
 
       fprintf(file, "%s:%lld %u\n",
               uuid_oss.str().c_str(), server_state.pause_seqno().get(),
-              wsrep_gtid_domain_id);
+              wsrep_gtid_server.domain_id);
       fsync(fileno(file));
       fclose(file);
       if (rename(tmp_name, real_name) == -1)
@@ -1655,13 +1625,11 @@ static void* sst_donor_thread (void* a)
   char         out_buf[out_len];
 
   wsrep_uuid_t  ret_uuid= WSREP_UUID_UNDEFINED;
-  // seqno of complete SST
-  wsrep_seqno_t ret_seqno= WSREP_SEQNO_UNDEFINED;
+  wsrep_seqno_t ret_seqno= WSREP_SEQNO_UNDEFINED; // seqno of complete SST
+
+  wsp::thd thd(FALSE); // we turn off wsrep_on for this THD so that it can
+                       // operate with wsrep_ready == OFF
 
-  // We turn off wsrep_on for this THD so that it can
-  // operate with wsrep_ready == OFF
-  // We also set this SST thread THD as system thread
-  wsp::thd thd(FALSE, true);
   wsp::process proc(arg->cmd, "r", arg->env);
 
   err= -proc.error();
@@ -1763,7 +1731,8 @@ wait_signal:
   proc.wait();
 
   wsrep_donor_monitor_end();
-  return nullptr;
+
+  return NULL;
 }
 
 static int sst_donate_other (const char*        method,
@@ -1797,8 +1766,6 @@ static int sst_donate_other (const char*        method,
   {
     WSREP_ERROR("sst_prepare_other(): generate_binlog_index_opt_val() failed %d",
                 ret);
-    if (binlog_opt_val) my_free(binlog_opt_val);
-    return ret;
   }
 
   make_wsrep_defaults_file();
@@ -1809,7 +1776,6 @@ static int sst_donate_other (const char*        method,
                  "wsrep_sst_%s "
                  WSREP_SST_OPT_ROLE " 'donor' "
                  WSREP_SST_OPT_ADDR " '%s' "
-                 WSREP_SST_OPT_LPORT " '%u' "
                  WSREP_SST_OPT_SOCKET " '%s' "
                  WSREP_SST_OPT_DATA " '%s' "
                  "%s"
@@ -1818,10 +1784,9 @@ static int sst_donate_other (const char*        method,
                  "%s"
                  "%s"
                  "%s",
-                 method, addr, mysqld_port, mysqld_unix_port,
-                 mysql_real_data_home,
+                 method, addr, mysqld_unix_port, mysql_real_data_home,
                  wsrep_defaults_file,
-                 uuid_oss.str().c_str(), gtid.seqno().get(), wsrep_gtid_domain_id,
+                 uuid_oss.str().c_str(), gtid.seqno().get(), wsrep_gtid_server.domain_id,
                  binlog_opt_val, binlog_index_opt_val,
                  bypass ? " " WSREP_SST_OPT_BYPASS : "");
 
@@ -1859,35 +1824,6 @@ static int sst_donate_other (const char*        method,
   return arg.err;
 }
 
-/* return true if character can be a part of a filename */
-static bool filename_char(int const c)
-{
-  return isalnum(c) || (c == '-') || (c == '_') || (c == '.');
-}
-
-/* return true if character can be a part of an address string */
-static bool address_char(int const c)
-{
-  return filename_char(c) ||
-         (c == ':') || (c == '[') || (c == ']') || (c == '/');
-}
-
-static bool check_request_str(const char* const str,
-                              bool (*check) (int c))
-{
-  for (size_t i(0); str[i] != '\0'; ++i)
-  {
-    if (!check(str[i]))
-    {
-      WSREP_WARN("Illegal character in state transfer request: %i (%c).",
-                 str[i], str[i]);
-      return true;
-    }
-  }
-
-  return false;
-}
-
 int wsrep_sst_donate(const std::string& msg,
                      const wsrep::gtid& current_gtid,
                      const bool         bypass)
@@ -1899,35 +1835,8 @@ int wsrep_sst_donate(const std::string& msg,
 
   const char* method= msg.data();
   size_t method_len= strlen (method);
-
-  if (check_request_str(method, filename_char))
-  {
-    WSREP_ERROR("Bad SST method name. SST canceled.");
-    return WSREP_CB_FAILURE;
-  }
-
   const char* data= method + method_len + 1;
 
-  /* check for auth@addr separator */
-  const char* addr= strrchr(data, '@');
-  wsp::string remote_auth;
-  if (addr)
-  {
-    remote_auth.set(strndup(data, addr - data));
-    addr++;
-  }
-  else
-  {
-    // no auth part
-    addr= data;
-  }
-
-  if (check_request_str(addr, address_char))
-  {
-    WSREP_ERROR("Bad SST address string. SST canceled.");
-    return WSREP_CB_FAILURE;
-  }
-
   wsp::env env(NULL);
   if (env.error())
   {
@@ -1936,25 +1845,15 @@ int wsrep_sst_donate(const std::string& msg,
   }
 
   int ret;
-  if ((ret= sst_append_env_var(env, WSREP_SST_AUTH_ENV, sst_auth_real)))
+  if ((ret= sst_append_auth_env(env, sst_auth_real)))
   {
     WSREP_ERROR("wsrep_sst_donate_cb(): appending auth env failed: %d", ret);
     return WSREP_CB_FAILURE;
   }
 
-  if (remote_auth())
-  {
-    if ((ret= sst_append_env_var(env, WSREP_SST_REMOTE_AUTH_ENV,remote_auth())))
-    {
-      WSREP_ERROR("wsrep_sst_donate_cb(): appending remote auth env failed: "
-                  "%d", ret);
-      return WSREP_CB_FAILURE;
-    }
-  }
-
   if (data_home_dir)
   {
-    if ((ret= sst_append_env_var(env, DATA_HOME_DIR_ENV, data_home_dir)))
+    if ((ret= sst_append_data_dir(env, data_home_dir)))
     {
       WSREP_ERROR("wsrep_sst_donate_cb(): appending data "
                   "directory failed: %d", ret);
@@ -1976,11 +1875,11 @@ int wsrep_sst_donate(const std::string& msg,
 
   if (!strcmp (WSREP_SST_MYSQLDUMP, method))
   {
-    ret= sst_donate_mysqldump(addr, current_gtid, bypass, env());
+    ret= sst_donate_mysqldump(data, current_gtid, bypass, env());
   }
   else
   {
-    ret= sst_donate_other(method, addr, current_gtid, bypass, env());
+    ret= sst_donate_other(method, data, current_gtid, bypass, env());
   }
 
   return (ret >= 0 ? 0 : 1);
