@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2012, Monty Program Ab
+   Copyright (c) 2012, 2020, MariaDB Corporation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -242,10 +242,9 @@ static int cassandra_init_func(void *p)
 
   cassandra_hton= (handlerton *)p;
   mysql_mutex_init(ex_key_mutex_example, &cassandra_mutex, MY_MUTEX_INIT_FAST);
-  (void) my_hash_init(&cassandra_open_tables,system_charset_info,32,0,0,
+  (void) my_hash_init(PSI_INSTRUMENT_ME, &cassandra_open_tables,system_charset_info,32,0,0,
                       (my_hash_get_key) cassandra_get_key,0,0);
 
-  cassandra_hton->state=   SHOW_OPTION_YES;
   cassandra_hton->create=  cassandra_create_handler;
   /*
     Don't specify HTON_CAN_RECREATE in flags. re-create is used by TRUNCATE
@@ -298,7 +297,7 @@ static CASSANDRA_SHARE *get_share(const char *table_name, TABLE *table)
                                               length)))
   {
     if (!(share=(CASSANDRA_SHARE *)
-          my_multi_malloc(MYF(MY_WME | MY_ZEROFILL),
+          my_multi_malloc(MYF(MY_WME | MY_ZEROFILL), PSI_INSTRUMENT_ME,
                           &share, sizeof(*share),
                           &tmp_name, length+1,
                           NullS)))
@@ -411,7 +410,7 @@ int ha_cassandra::check_field_options(Field **fields)
     {
       if (dyncol_set || (*field)->type() != MYSQL_TYPE_BLOB)
       {
-         my_error(ER_WRONG_FIELD_SPEC, MYF(0), (*field)->field_name);
+         my_error(ER_WRONG_FIELD_SPEC, MYF(0), (*field)->field_name.str);
          DBUG_RETURN(HA_WRONG_CREATE_OPTION);
       }
       dyncol_set= 1;
@@ -866,7 +865,7 @@ static void alloc_strings_memroot(MEM_ROOT *mem_root)
       The mem_root used to allocate UUID (of length 36 + \0) so make
       appropriate allocated size
     */
-    init_alloc_root(mem_root,
+    init_alloc_root(PSI_INSTRUMENT_ME, mem_root,
                     (36 + 1 + ALIGN_SIZE(sizeof(USED_MEM))) * 10 +
                     ALLOC_ROOT_MIN_BLOCK_SIZE,
                     (36 + 1 + ALIGN_SIZE(sizeof(USED_MEM))) * 10 +
@@ -1105,7 +1104,7 @@ bool cassandra_to_dyncol_strUTF8(const char *cass_data,
                                  MEM_ROOT *mem_root __attribute__((unused)))
 {
   return cassandra_to_dyncol_strStr(cass_data, cass_data_len, value,
-                                    &my_charset_utf8_unicode_ci);
+                                    &my_charset_utf8mb3_unicode_ci);
 }
 
 bool dyncol_to_cassandraUTF8(DYNAMIC_COLUMN_VALUE *value,
@@ -1113,7 +1112,7 @@ bool dyncol_to_cassandraUTF8(DYNAMIC_COLUMN_VALUE *value,
                              void* buff, void **freemem)
 {
   return dyncol_to_cassandraStr(value, cass_data, cass_data_len,
-                                buff, freemem, &my_charset_utf8_unicode_ci);
+                                buff, freemem, &my_charset_utf8mb3_unicode_ci);
 }
 
 bool cassandra_to_dyncol_strUUID(const char *cass_data,
@@ -1447,7 +1446,7 @@ bool ha_cassandra::setup_field_converters(Field **field_arg, uint n_fields)
   size_t memsize= sizeof(ColumnDataConverter*) * n_fields +
     (sizeof(LEX_STRING) + sizeof(CASSANDRA_TYPE_DEF))*
     (dyncol_set ? max_non_default_fields : 0);
-  if (!(field_converters= (ColumnDataConverter**)my_malloc(memsize, MYF(0))))
+  if (!(field_converters= (ColumnDataConverter**)my_malloc(PSI_INSTRUMENT_ME, memsize, MYF(0))))
     DBUG_RETURN(true);
   bzero(field_converters, memsize);
   n_field_converters= n_fields;
@@ -1459,12 +1458,12 @@ bool ha_cassandra::setup_field_converters(Field **field_arg, uint n_fields)
     special_type_field_names=
       ((LEX_STRING*)(special_type_field_converters + max_non_default_fields));
 
-    if (my_init_dynamic_array(&dynamic_values,
+    if (my_init_dynamic_array(PSI_INSTRUMENT_ME, &dynamic_values,
                            sizeof(DYNAMIC_COLUMN_VALUE),
                            DYNCOL_USUAL, DYNCOL_DELTA, MYF(0)))
       DBUG_RETURN(true);
     else
-      if (my_init_dynamic_array(&dynamic_names,
+      if (my_init_dynamic_array(PSI_INSTRUMENT_ME, &dynamic_names,
                              sizeof(LEX_STRING),
                              DYNCOL_USUAL, DYNCOL_DELTA,MYF(0)))
       {
@@ -1497,14 +1496,14 @@ bool ha_cassandra::setup_field_converters(Field **field_arg, uint n_fields)
     for (field= field_arg + 1, i= 1; *field; field++, i++)
     {
       if ((!dyncol_set || dyncol_field != i) &&
-          !strcmp((*field)->field_name, col_name))
+          !strcmp((*field)->field_name.str, col_name))
       {
         n_mapped++;
         ColumnDataConverter **conv= field_converters + (*field)->field_index;
         if (!(*conv= map_field_to_validator(*field, col_type)))
         {
           se->print_error("Failed to map column %s to datatype %s",
-                          (*field)->field_name, col_type);
+                          (*field)->field_name.str, col_type);
           my_error(ER_INTERNAL_ERROR, MYF(0), se->error_str());
           DBUG_RETURN(true);
         }
@@ -1543,7 +1542,7 @@ bool ha_cassandra::setup_field_converters(Field **field_arg, uint n_fields)
     DBUG_ASSERT(first_unmapped);
 
     se->print_error("Field `%s` could not be mapped to any field in Cassandra",
-                    first_unmapped->field_name);
+                    first_unmapped->field_name.str);
     my_error(ER_INTERNAL_ERROR, MYF(0), se->error_str());
     DBUG_RETURN(true);
   }
@@ -1552,14 +1551,14 @@ bool ha_cassandra::setup_field_converters(Field **field_arg, uint n_fields)
     Setup type conversion for row_key.
   */
   se->get_rowkey_type(&col_name, &col_type);
-  if (col_name && strcmp(col_name, (*field_arg)->field_name))
+  if (col_name && strcmp(col_name, (*field_arg)->field_name.str))
   {
     se->print_error("PRIMARY KEY column must match Cassandra's name '%s'",
                     col_name);
     my_error(ER_INTERNAL_ERROR, MYF(0), se->error_str());
     DBUG_RETURN(true);
   }
-  if (!col_name && strcmp("rowkey", (*field_arg)->field_name))
+  if (!col_name && strcmp("rowkey", (*field_arg)->field_name.str))
   {
     se->print_error("target column family has no key_alias defined, "
                     "PRIMARY KEY column must be named 'rowkey'");
@@ -1641,18 +1640,18 @@ int ha_cassandra::index_read_map(uchar *buf, const uchar *key,
 
   char *cass_key;
   int cass_key_len;
-  MY_BITMAP *old_map;
+  my_bitmap_map *old_map;
 
-  old_map= dbug_tmp_use_all_columns(table, &table->read_set);
+  old_map= dbug_tmp_use_all_columns(table, table->read_set);
 
   if (rowkey_converter->mariadb_to_cassandra(&cass_key, &cass_key_len))
   {
     /* We get here when making lookups like uuid_column='not-an-uuid' */
-    dbug_tmp_restore_column_map(&table->read_set, old_map);
+    dbug_tmp_restore_column_map(table->read_set, old_map);
     DBUG_RETURN(HA_ERR_KEY_NOT_FOUND);
   }
 
-  dbug_tmp_restore_column_map(&table->read_set, old_map);
+  dbug_tmp_restore_column_map(table->read_set, old_map);
 
   bool found;
   if (se->get_slice(cass_key, cass_key_len, &found))
@@ -1726,8 +1725,8 @@ int ha_cassandra::read_cassandra_columns(bool unpack_pk)
     cassandra_to_mariadb() calls will use field->store(...) methods, which
     require that the column is in the table->write_set
   */
-  MY_BITMAP *old_map;
-  old_map= dbug_tmp_use_all_columns(table, &table->write_set);
+  my_bitmap_map *old_map;
+  old_map= dbug_tmp_use_all_columns(table, table->write_set);
 
   /* Start with all fields being NULL */
   for (field= table->field + 1; *field; field++)
@@ -1742,14 +1741,14 @@ int ha_cassandra::read_cassandra_columns(bool unpack_pk)
     {
       uint fieldnr= (*field)->field_index;
       if ((!dyncol_set || dyncol_field != fieldnr) &&
-          !strcmp((*field)->field_name, cass_name))
+          !strcmp((*field)->field_name.str, cass_name))
       {
         found= 1;
         (*field)->set_notnull();
         if (field_converters[fieldnr]->cassandra_to_mariadb(cass_value,
                                                             cass_value_len))
         {
-          print_conversion_error((*field)->field_name, cass_value,
+          print_conversion_error((*field)->field_name.str, cass_value,
                                  cass_value_len);
           res=1;
           goto err;
@@ -1770,7 +1769,7 @@ int ha_cassandra::read_cassandra_columns(bool unpack_pk)
         se->print_error("Unable to convert value for field `%s`"
                         " from Cassandra's data format. Name"
                         " length exceed limit of %u: '%s'",
-                        table->field[dyncol_field]->field_name,
+                        table->field[dyncol_field]->field_name.str,
                         (uint)MAX_NAME_LENGTH, cass_name);
         my_error(ER_INTERNAL_ERROR, MYF(0), se->error_str());
         res=1;
@@ -1782,7 +1781,7 @@ int ha_cassandra::read_cassandra_columns(bool unpack_pk)
         se->print_error("Unable to convert value for field `%s`"
                         " from Cassandra's data format. Sum of all names"
                         " length exceed limit of %lu",
-                        table->field[dyncol_field]->field_name,
+                        table->field[dyncol_field]->field_name.str,
                         cass_name, (uint)MAX_TOTAL_NAME_LENGTH);
         my_error(ER_INTERNAL_ERROR, MYF(0), se->error_str());
         res=1;
@@ -1841,14 +1840,14 @@ int ha_cassandra::read_cassandra_columns(bool unpack_pk)
     se->get_read_rowkey(&cass_value, &cass_value_len);
     if (rowkey_converter->cassandra_to_mariadb(cass_value, cass_value_len))
     {
-      print_conversion_error((*field)->field_name, cass_value, cass_value_len);
+      print_conversion_error((*field)->field_name.str, cass_value, cass_value_len);
       res=1;
       goto err;
     }
   }
 
 err:
-  dbug_tmp_restore_column_map(&table->write_set, old_map);
+  dbug_tmp_restore_column_map(table->write_set, old_map);
   return res;
 }
 
@@ -1931,9 +1930,9 @@ void ha_cassandra::free_dynamic_row(DYNAMIC_COLUMN_VALUE **vals,
   *names= 0;
 }
 
-int ha_cassandra::write_row(uchar *buf)
+int ha_cassandra::write_row(const uchar *buf)
 {
-  MY_BITMAP *old_map;
+  my_bitmap_map *old_map;
   int ires;
   DBUG_ENTER("ha_cassandra::write_row");
 
@@ -1943,7 +1942,7 @@ int ha_cassandra::write_row(uchar *buf)
   if (!doing_insert_batch)
     se->clear_insert_buffer();
 
-  old_map= dbug_tmp_use_all_columns(table, &table->read_set);
+  old_map= dbug_tmp_use_all_columns(table, table->read_set);
 
   insert_lineno++;
 
@@ -1953,8 +1952,8 @@ int ha_cassandra::write_row(uchar *buf)
   if (rowkey_converter->mariadb_to_cassandra(&cass_key, &cass_key_len))
   {
     my_error(ER_WARN_DATA_OUT_OF_RANGE, MYF(0),
-             rowkey_converter->field->field_name, insert_lineno);
-    dbug_tmp_restore_column_map(&table->read_set, old_map);
+             rowkey_converter->field->field_name.str, insert_lineno);
+    dbug_tmp_restore_column_map(table->read_set, old_map);
     DBUG_RETURN(HA_ERR_INTERNAL_ERROR);
   }
   se->start_row_insert(cass_key, cass_key_len);
@@ -1977,7 +1976,7 @@ int ha_cassandra::write_row(uchar *buf)
       free_dynamic_row(&vals, &names);
       if (rc)
       {
-        dbug_tmp_restore_column_map(&table->read_set, old_map);
+        dbug_tmp_restore_column_map(table->read_set, old_map);
         DBUG_RETURN(rc);
       }
     }
@@ -1987,16 +1986,16 @@ int ha_cassandra::write_row(uchar *buf)
                                                     &cass_data_len))
       {
         my_error(ER_WARN_DATA_OUT_OF_RANGE, MYF(0),
-                 field_converters[i]->field->field_name, insert_lineno);
-        dbug_tmp_restore_column_map(&table->read_set, old_map);
+                 field_converters[i]->field->field_name.str, insert_lineno);
+        dbug_tmp_restore_column_map(table->read_set, old_map);
         DBUG_RETURN(HA_ERR_INTERNAL_ERROR);
       }
-      se->add_insert_column(field_converters[i]->field->field_name, 0,
+      se->add_insert_column(field_converters[i]->field->field_name.str, 0,
                             cass_data, cass_data_len);
     }
   }
 
-  dbug_tmp_restore_column_map(&table->read_set, old_map);
+  dbug_tmp_restore_column_map(table->read_set, old_map);
 
   bool res;
 
@@ -2074,7 +2073,7 @@ int ha_cassandra::rnd_init(bool scan)
   {
     se->clear_read_columns();
     for (uint i= 1; i < table->s->fields; i++)
-      se->add_read_column(table->field[i]->field_name);
+      se->add_read_column(table->field[i]->field_name.str);
   }
 
   se->read_batch_size= THDVAR(table->in_use, rnd_batch_size);
@@ -2171,7 +2170,7 @@ int ha_cassandra::info(uint flag)
 }
 
 
-void key_copy(uchar *to_key, uchar *from_record, KEY *key_info,
+void key_copy(uchar *to_key, const uchar *from_record, const KEY *key_info,
               uint key_length, bool with_zerofill);
 
 
@@ -2263,8 +2262,8 @@ bool ha_cassandra::mrr_start_read()
 {
   uint key_len;
 
-  MY_BITMAP *old_map;
-  old_map= dbug_tmp_use_all_columns(table, &table->read_set);
+  my_bitmap_map *old_map;
+  old_map= dbug_tmp_use_all_columns(table, table->read_set);
 
   se->new_lookup_keys();
 
@@ -2288,7 +2287,7 @@ bool ha_cassandra::mrr_start_read()
       break;
   }
 
-  dbug_tmp_restore_column_map(&table->read_set, old_map);
+  dbug_tmp_restore_column_map(table->read_set, old_map);
 
   return se->multiget_slice();
 }
@@ -2355,18 +2354,18 @@ public:
     if (idx == obj->table->s->fields)
       return NULL;
     else
-      return obj->table->field[idx++]->field_name;
+      return obj->table->field[idx++]->field_name.str;
   }
 };
 
 
-int ha_cassandra::update_row(const uchar *old_data, uchar *new_data)
+int ha_cassandra::update_row(const uchar *old_data, const uchar *new_data)
 {
   DYNAMIC_COLUMN_VALUE *oldvals, *vals;
   LEX_STRING *oldnames, *names;
   uint oldcount, count;
   String oldvalcol, valcol;
-  MY_BITMAP *old_map;
+  my_bitmap_map *old_map;
   int res;
   DBUG_ENTER("ha_cassandra::update_row");
   /* Currently, it is guaranteed that new_data == table->record[0] */
@@ -2374,7 +2373,7 @@ int ha_cassandra::update_row(const uchar *old_data, uchar *new_data)
   /* For now, just rewrite the full record */
   se->clear_insert_buffer();
 
-  old_map= dbug_tmp_use_all_columns(table, &table->read_set);
+  old_map= dbug_tmp_use_all_columns(table, table->read_set);
 
   char *old_key;
   int old_key_len;
@@ -2386,8 +2385,8 @@ int ha_cassandra::update_row(const uchar *old_data, uchar *new_data)
   if (rowkey_converter->mariadb_to_cassandra(&new_key, &new_key_len))
   {
     my_error(ER_WARN_DATA_OUT_OF_RANGE, MYF(0),
-             rowkey_converter->field->field_name, insert_lineno);
-    dbug_tmp_restore_column_map(&table->read_set, old_map);
+             rowkey_converter->field->field_name.str, insert_lineno);
+    dbug_tmp_restore_column_map(table->read_set, old_map);
     DBUG_RETURN(HA_ERR_INTERNAL_ERROR);
   }
 
@@ -2449,11 +2448,11 @@ int ha_cassandra::update_row(const uchar *old_data, uchar *new_data)
       if (field_converters[i]->mariadb_to_cassandra(&cass_data, &cass_data_len))
       {
         my_error(ER_WARN_DATA_OUT_OF_RANGE, MYF(0),
-                 field_converters[i]->field->field_name, insert_lineno);
-        dbug_tmp_restore_column_map(&table->read_set, old_map);
+                 field_converters[i]->field->field_name.str, insert_lineno);
+        dbug_tmp_restore_column_map(table->read_set, old_map);
         DBUG_RETURN(HA_ERR_INTERNAL_ERROR);
       }
-      se->add_insert_column(field_converters[i]->field->field_name, 0,
+      se->add_insert_column(field_converters[i]->field->field_name.str, 0,
                             cass_data, cass_data_len);
     }
   }
@@ -2477,7 +2476,7 @@ int ha_cassandra::update_row(const uchar *old_data, uchar *new_data)
     }
   }
 
-  dbug_tmp_restore_column_map(&table->read_set, old_map);
+  dbug_tmp_restore_column_map(table->read_set, old_map);
 
   res= se->do_insert();
 
@@ -2528,14 +2527,6 @@ THR_LOCK_DATA **ha_cassandra::store_lock(THD *thd,
 }
 
 
-ha_rows ha_cassandra::records_in_range(uint inx, key_range *min_key,
-                                       key_range *max_key)
-{
-  DBUG_ENTER("ha_cassandra::records_in_range");
-  DBUG_RETURN(HA_POS_ERROR); /* Range scans are not supported */
-}
-
-
 /**
   check_if_incompatible_data() called if ALTER TABLE can't detect otherwise
   if new and old definition are compatible
