@@ -616,10 +616,11 @@ dtuple_convert_big_rec(
 	stored externally */
 
 	n_fields = 0;
-	ulint longest_i;
+	uint16_t longest_i;
+	ulint longest;
 
 	const bool mblob = entry->is_alter_metadata();
-	ut_ad(entry->n_fields >= index->first_user_field() + mblob);
+	ut_ad(entry->n_fields - mblob >= index->first_user_field());
 	ut_ad(entry->n_fields - mblob <= index->n_fields);
 
 	if (mblob) {
@@ -645,8 +646,9 @@ dtuple_convert_big_rec(
 				      dict_index_get_n_fields(index),
 				      zip_size)) {
 		longest_i = 0;
-		for (ulint i = index->first_user_field(), longest = 0;
-		     i + mblob < entry->n_fields; i++) {
+		longest = 0;
+		for (uint16_t i = index->first_user_field();
+		     i < entry->n_fields - mblob; i++) {
 			ulint	savings;
 			dfield = dtuple_get_nth_field(entry, i + mblob);
 
@@ -686,7 +688,7 @@ dtuple_convert_big_rec(
 				goto skip_field;
 			}
 
-			longest_i = i + mblob;
+			longest_i = i;
 			longest = savings;
 
 skip_field:
@@ -738,7 +740,7 @@ ext_write:
 			DEBUG_SYNC_C("ib_mv_nonupdated_column_offpage");
 
 			upd_field_t	upd_field;
-			upd_field.field_no = unsigned(longest_i);
+			upd_field.field_no = longest_i;
 			upd_field.orig_len = 0;
 			upd_field.exp = NULL;
 			upd_field.old_v_val = NULL;
@@ -767,7 +769,7 @@ void
 dtuple_convert_back_big_rec(
 /*========================*/
 	dict_index_t*	index MY_ATTRIBUTE((unused)),	/*!< in: index */
-	dtuple_t*	entry,	/*!< in/out: entry whose data was put to vector */
+	dtuple_t*	entry,	/*!< in: entry whose data was put to vector */
 	big_rec_t*	vector)	/*!< in, own: big rec vector; it is
 				freed in this function */
 {
