@@ -1,6 +1,6 @@
 /*****************************************************************************
 Copyright (C) 2013, 2015, Google Inc. All Rights Reserved.
-Copyright (c) 2015, 2019, MariaDB Corporation.
+Copyright (c) 2015, 2020, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -93,11 +93,6 @@ struct fil_space_rotate_state_t
 				space */
 	bool starting;		/*!< initial write of IV */
 	bool flushing;		/*!< space is being flushed at end of rotate */
-	struct {
-		bool is_active; /*!< is scrubbing active in this space */
-		time_t last_scrub_completed; /*!< when was last scrub
-					     completed */
-	} scrubbing;
 };
 
 #ifndef UNIV_INNOCHECKSUM
@@ -115,7 +110,6 @@ struct fil_space_crypt_t : st_encryption_scheme
 		fil_encryption_t new_encryption)
 		: st_encryption_scheme(),
 		min_key_version(new_min_key_version),
-		page0_offset(0),
 		encryption(new_encryption),
 		key_found(0),
 		rotate_state()
@@ -184,14 +178,12 @@ struct fil_space_crypt_t : st_encryption_scheme
 	@param[in,out]	page	first page of the tablespace */
 	void fill_page0(ulint flags, byte* page);
 
-	/** Write crypt data to a page (0)
-	@param[in]	space	tablespace
-	@param[in,out]	page0	first page of the tablespace
+	/** Write encryption metadata to the first page.
+	@param[in,out]	block	first page of the tablespace
 	@param[in,out]	mtr	mini-transaction */
-	void write_page0(const fil_space_t* space, byte* page0, mtr_t* mtr);
+	void write_page0(buf_block_t* block, mtr_t* mtr);
 
 	uint min_key_version; // min key version for this space
-	ulint page0_offset;   // byte offset on page 0 for crypt data
 	fil_encryption_t encryption; // Encryption setup
 
 	ib_mutex_t mutex;   // mutex protecting following variables
@@ -229,18 +221,6 @@ struct fil_crypt_stat_t {
 	ulint estimated_iops;
 };
 
-/** Status info about scrubbing */
-struct fil_space_scrub_status_t {
-	ulint space;             /*!< tablespace id */
-	bool compressed;        /*!< is space compressed  */
-	time_t last_scrub_completed;  /*!< when was last scrub completed */
-	bool scrubbing;               /*!< is scrubbing ongoing */
-	time_t current_scrub_started; /*!< when started current scrubbing */
-	ulint current_scrub_active_threads; /*!< current scrub active threads */
-	ulint current_scrub_page_number; /*!< current scrub page no */
-	ulint current_scrub_max_page_number; /*!< current scrub max page no */
-};
-
 /*********************************************************************
 Init space crypt */
 UNIV_INTERN
@@ -294,25 +274,15 @@ void
 fil_space_destroy_crypt_data(
 	fil_space_crypt_t **crypt_data);
 
-/******************************************************************
-Parse a MLOG_FILE_WRITE_CRYPT_DATA log entry
-@param[in]	ptr		Log entry start
-@param[in]	end_ptr		Log entry end
-@param[out]	err		DB_SUCCESS or DB_DECRYPTION_FAILED
-@return position on log buffer */
-UNIV_INTERN
-byte*
-fil_parse_write_crypt_data(
-	byte*			ptr,
-	const byte*		end_ptr,
-	dberr_t*		err)
-	MY_ATTRIBUTE((warn_unused_result));
+/** Amend encryption information from redo log.
+@param[in]	space	tablespace
+@param[in]	data	encryption metadata */
+void fil_crypt_parse(fil_space_t* space, const byte* data);
 
 /** Encrypt a buffer.
 @param[in,out]		crypt_data		Crypt data
 @param[in]		space			space_id
 @param[in]		offset			Page offset
-@param[in]		lsn			Log sequence number
 @param[in]		src_frame		Page to encrypt
 @param[in]		zip_size		ROW_FORMAT=COMPRESSED page size, or 0
 @param[in,out]		dst_frame		Output buffer
@@ -324,7 +294,6 @@ fil_encrypt_buf(
 	fil_space_crypt_t*	crypt_data,
 	ulint			space,
 	ulint			offset,
-	lsn_t			lsn,
 	const byte*		src_frame,
 	ulint			zip_size,
 	byte*			dst_frame,
@@ -336,16 +305,12 @@ Encrypt a page.
 
 @param[in]		space		Tablespace
 @param[in]		offset		Page offset
-@param[in]		lsn		Log sequence number
 @param[in]		src_frame	Page to encrypt
 @param[in,out]		dst_frame	Output buffer
 @return encrypted buffer or NULL */
-UNIV_INTERN
-byte*
-fil_space_encrypt(
+byte* fil_space_encrypt(
 	const fil_space_t* space,
 	ulint		offset,
-	lsn_t		lsn,
 	byte*		src_frame,
 	byte*		dst_frame)
 	MY_ATTRIBUTE((warn_unused_result));
@@ -423,10 +388,7 @@ fil_crypt_set_rotation_iops(
 /*********************************************************************
 Adjust encrypt tables
 @param[in]	val		New setting for innodb-encrypt-tables */
-UNIV_INTERN
-void
-fil_crypt_set_encrypt_tables(
-	uint val);
+void fil_crypt_set_encrypt_tables(ulong val);
 
 /*********************************************************************
 Init threads for key rotation */
@@ -467,18 +429,6 @@ void
 fil_crypt_total_stat(
 	fil_crypt_stat_t *stat);
 
-/**
-Get scrub status for a space (used by information_schema)
-
-@param[in]	space		Tablespace
-@param[out]	status		Scrub status
-return 0 if data found */
-UNIV_INTERN
-void
-fil_space_get_scrub_status(
-	const fil_space_t*		space,
-	fil_space_scrub_status_t*	status);
-
 #include "fil0crypt.ic"
 #endif /* !UNIV_INNOCHECKSUM */
 
@@ -496,10 +446,4 @@ encrypted, or corrupted.
 bool fil_space_verify_crypt_checksum(const byte* page, ulint zip_size)
 	MY_ATTRIBUTE((warn_unused_result));
 
-/** Add the tablespace to the rotation list if
-innodb_encrypt_rotate_key_age is 0 or encryption plugin does
-not do key version rotation
-@return whether the tablespace should be added to rotation list */
-bool fil_crypt_must_default_encrypt();
-
 #endif /* fil0crypt_h */
