@@ -398,10 +398,7 @@ enum monitor_id_t {
 	MONITOR_MASTER_ACTIVE_LOOPS,
 	MONITOR_MASTER_IDLE_LOOPS,
 	MONITOR_SRV_BACKGROUND_DROP_TABLE_MICROSECOND,
-	MONITOR_SRV_IBUF_MERGE_MICROSECOND,
 	MONITOR_SRV_LOG_FLUSH_MICROSECOND,
-	MONITOR_SRV_MEM_VALIDATE_MICROSECOND,
-	MONITOR_SRV_PURGE_MICROSECOND,
 	MONITOR_SRV_DICT_LRU_MICROSECOND,
 	MONITOR_SRV_DICT_LRU_EVICT_COUNT_ACTIVE,
 	MONITOR_SRV_DICT_LRU_EVICT_COUNT_IDLE,
@@ -498,23 +495,23 @@ enum mon_option_t {
 
 /** This "monitor_set_tbl" is a bitmap records whether a particular monitor
 counter has been turned on or off */
-extern Atomic_relaxed<ulint>
-    monitor_set_tbl[(NUM_MONITOR + NUM_BITS_ULINT - 1) / NUM_BITS_ULINT];
+extern ulint		monitor_set_tbl[(NUM_MONITOR + NUM_BITS_ULINT - 1) /
+					NUM_BITS_ULINT];
 
 /** Macros to turn on/off the control bit in monitor_set_tbl for a monitor
 counter option. */
-#define MONITOR_ON(monitor)                                                   \
-  (monitor_set_tbl[unsigned(monitor) / NUM_BITS_ULINT].fetch_or(              \
-      (ulint(1) << (unsigned(monitor) % NUM_BITS_ULINT))))
+#define MONITOR_ON(monitor)					\
+	(monitor_set_tbl[unsigned(monitor) / NUM_BITS_ULINT] |=	\
+	 (ulint(1) << (unsigned(monitor) % NUM_BITS_ULINT)))
 
-#define MONITOR_OFF(monitor)                                                  \
-  (monitor_set_tbl[unsigned(monitor) / NUM_BITS_ULINT].fetch_and(             \
-      ~(ulint(1) << (unsigned(monitor) % NUM_BITS_ULINT))))
+#define MONITOR_OFF(monitor)					\
+	(monitor_set_tbl[unsigned(monitor) / NUM_BITS_ULINT] &=	\
+	 ~(ulint(1) << (unsigned(monitor) % NUM_BITS_ULINT)))
 
 /** Check whether the requested monitor is turned on/off */
-#define MONITOR_IS_ON(monitor)                                                \
-  (monitor_set_tbl[unsigned(monitor) / NUM_BITS_ULINT] &                      \
-   (ulint(1) << (unsigned(monitor) % NUM_BITS_ULINT)))
+#define MONITOR_IS_ON(monitor)					\
+	(monitor_set_tbl[unsigned(monitor) / NUM_BITS_ULINT] &	\
+	 (ulint(1) << (unsigned(monitor) % NUM_BITS_ULINT)))
 
 /** The actual monitor counter array that records each monintor counter
 value */
