@@ -158,7 +158,7 @@ int my_strcasecmp_mb(CHARSET_INFO * cs,const char *s, const char *t)
         if (*s++ != *t++) 
           return 1;
     }
-    else if (my_charlen(cs, t, t + cs->mbmaxlen) > 1)
+    else if (my_ci_charlen(cs, (const uchar *) t, (const uchar *) t + cs->mbmaxlen) > 1)
       return 1;
     else if (map[(uchar) *s++] != map[(uchar) *t++])
       return 1;
@@ -354,8 +354,8 @@ my_append_fix_badly_formed_tail(CHARSET_INFO *cs,
   for ( ; nchars; nchars--)
   {
     int chlen;
-    if ((chlen= cs->cset->charlen(cs, (const uchar*) from,
-                                      (const uchar *) from_end)) > 0)
+    if ((chlen= my_ci_charlen(cs, (const uchar*) from,
+                                  (const uchar *) from_end)) > 0)
     {
       /* Found a valid character */         /* chlen == 1..MBMAXLEN  */
       DBUG_ASSERT(chlen <= (int) cs->mbmaxlen);
@@ -381,7 +381,7 @@ bad:
     if (!status->m_well_formed_error_pos)
       status->m_well_formed_error_pos= from;
 
-    if ((chlen= cs->cset->wc_mb(cs, '?', (uchar*) to, (uchar *) to_end)) <= 0)
+    if ((chlen= my_ci_wc_mb(cs, '?', (uchar*) to, (uchar *) to_end)) <= 0)
       break; /* Question mark does not fit into the destination */
     to+= chlen;
     from++;
@@ -403,13 +403,10 @@ my_copy_fix_mb(CHARSET_INFO *cs,
   size_t fixed_length;
 
   set_if_smaller(src_length, dst_length);
-  well_formed_nchars= cs->cset->well_formed_char_length(cs,
-                                                        src, src + src_length,
+  well_formed_nchars= my_ci_well_formed_char_length(cs, src, src + src_length,
                                                         nchars, status);
   DBUG_ASSERT(well_formed_nchars <= nchars);
-  well_formed_length= status->m_source_end_pos - src;
-  if (well_formed_length)
-    memmove(dst, src, well_formed_length);
+  memmove(dst, src, (well_formed_length= status->m_source_end_pos - src));
   if (!status->m_well_formed_error_pos)
     return well_formed_length;
 
@@ -452,8 +449,8 @@ uint my_instr_mb(CHARSET_INFO *cs,
     {
       int mb_len;
       
-      if (!cs->coll->strnncoll(cs, (uchar*) b,   s_length, 
-      				   (uchar*) s, s_length, 0))
+      if (!my_ci_strnncoll(cs, (const uchar *) b, s_length,
+                               (const uchar *) s, s_length, 0))
       {
         if (nmatch)
         {
@@ -662,14 +659,14 @@ my_hash_sort_mb_bin(CHARSET_INFO *cs __attribute__((unused)),
 static void pad_max_char(CHARSET_INFO *cs, char *str, char *end)
 {
   char buf[10];
-  char buflen= cs->cset->native_to_mb(cs, cs->max_sort_char, (uchar*) buf,
+  char buflen= my_ci_native_to_mb(cs, cs->max_sort_char, (uchar*) buf,
                                       (uchar*) buf + sizeof(buf));
   DBUG_ASSERT(buflen > 0);
   do
   {
     if ((str + buflen) <= end)
     {
-      /* Enough space for the characer */
+      /* Enough space for the character */
       memcpy(str, buf, buflen);
       str+= buflen;
     }
@@ -877,7 +874,7 @@ my_like_range_generic(CHARSET_INFO *cs,
   {
     my_wc_t wc, wc2;
     int res;
-    if ((res= cs->cset->mb_wc(cs, &wc, (uchar*) ptr, (uchar*) end)) <= 0)
+    if ((res= my_ci_mb_wc(cs, &wc, (uchar*) ptr, (uchar*) end)) <= 0)
     {
       if (res == MY_CS_ILSEQ) /* Bad sequence */
         return TRUE; /* min_length and max_length are not important */
@@ -887,7 +884,7 @@ my_like_range_generic(CHARSET_INFO *cs,
 
     if (wc == (my_wc_t) escape)
     {
-      if ((res= cs->cset->mb_wc(cs, &wc, (uchar*) ptr, (uchar*) end)) <= 0)
+      if ((res= my_ci_mb_wc(cs, &wc, (uchar*) ptr, (uchar*) end)) <= 0)
       {
         if (res == MY_CS_ILSEQ)
           return TRUE; /* min_length and max_length are not important */
@@ -901,26 +898,24 @@ my_like_range_generic(CHARSET_INFO *cs,
         ptr+= res;
 
       /* Put escape character to min_str and max_str  */
-      if ((res= cs->cset->wc_mb(cs, wc,
-                                (uchar*) min_str, (uchar*) min_end)) <= 0)
+      if ((res= my_ci_wc_mb(cs, wc, (uchar*) min_str, (uchar*) min_end)) <= 0)
         goto pad_set_lengths; /* No space */
       min_str+= res;
 
-      if ((res= cs->cset->wc_mb(cs, wc,
-                                (uchar*) max_str, (uchar*) max_end)) <= 0)
+      if ((res= my_ci_wc_mb(cs, wc, (uchar*) max_str, (uchar*) max_end)) <= 0)
         goto pad_set_lengths; /* No space */
       max_str+= res;
       continue;
     }
     else if (wc == (my_wc_t) w_one)
     {
-      if ((res= cs->cset->wc_mb(cs, cs->min_sort_char,
-                                (uchar*) min_str, (uchar*) min_end)) <= 0)
+      if ((res= my_ci_wc_mb(cs, cs->min_sort_char,
+                             (uchar*) min_str, (uchar*) min_end)) <= 0)
         goto pad_set_lengths;
       min_str+= res;
 
-      if ((res= cs->cset->wc_mb(cs, cs->max_sort_char,
-                                (uchar*) max_str, (uchar*) max_end)) <= 0)
+      if ((res= my_ci_wc_mb(cs, cs->max_sort_char,
+                             (uchar*) max_str, (uchar*) max_end)) <= 0)
         goto pad_set_lengths;
       max_str+= res;
       continue;
@@ -941,7 +936,7 @@ my_like_range_generic(CHARSET_INFO *cs,
 
     if (contractions &&
         my_uca_can_be_contraction_head(contractions, wc) &&
-        (res= cs->cset->mb_wc(cs, &wc2, (uchar*) ptr, (uchar*) end)) > 0)
+        (res= my_ci_mb_wc(cs, &wc2, (uchar*) ptr, (uchar*) end)) > 0)
     {
       const uint16 *weight;
       if ((wc2 == (my_wc_t) w_one || wc2 == (my_wc_t) w_many))
@@ -966,13 +961,11 @@ my_like_range_generic(CHARSET_INFO *cs,
         charlen--;
 
         /* Put contraction head */
-        if ((res= cs->cset->wc_mb(cs, wc,
-                                  (uchar*) min_str, (uchar*) min_end)) <= 0)
+        if ((res= my_ci_wc_mb(cs, wc, (uchar*) min_str, (uchar*) min_end)) <= 0)
           goto pad_set_lengths;
         min_str+= res;
 
-        if ((res= cs->cset->wc_mb(cs, wc,
-                                  (uchar*) max_str, (uchar*) max_end)) <= 0)
+        if ((res= my_ci_wc_mb(cs, wc, (uchar*) max_str, (uchar*) max_end)) <= 0)
           goto pad_set_lengths;
         max_str+= res;
         wc= wc2; /* Prepare to put contraction tail */
@@ -980,12 +973,10 @@ my_like_range_generic(CHARSET_INFO *cs,
     }
 
     /* Normal character, or contraction tail */
-    if ((res= cs->cset->wc_mb(cs, wc,
-                              (uchar*) min_str, (uchar*) min_end)) <= 0)
+    if ((res= my_ci_wc_mb(cs, wc, (uchar*) min_str, (uchar*) min_end)) <= 0)
       goto pad_set_lengths;
     min_str+= res;
-    if ((res= cs->cset->wc_mb(cs, wc,
-                              (uchar*) max_str, (uchar*) max_end)) <= 0)
+    if ((res= my_ci_wc_mb(cs, wc, (uchar*) max_str, (uchar*) max_end)) <= 0)
       goto pad_set_lengths;
     max_str+= res;
   }
@@ -1002,9 +993,9 @@ pad_min_max:
     Make sure to call fill() with proper "length" argument.
   */
   res_length_diff= res_length % cs->mbminlen;
-  cs->cset->fill(cs, min_str, min_end - min_str - res_length_diff,
+  my_ci_fill(cs, min_str, min_end - min_str - res_length_diff,
                  cs->min_sort_char);
-  cs->cset->fill(cs, max_str, max_end - max_str - res_length_diff,
+  my_ci_fill(cs, max_str, max_end - max_str - res_length_diff,
                  cs->max_sort_char);
 
   /* In case of incomplete characters set the remainder to 0x00's */
@@ -1356,7 +1347,7 @@ size_t my_numcells_mb(CHARSET_INFO *cs, const char *b, const char *e)
   {
     int mb_len;
     uint pg;
-    if ((mb_len= cs->cset->mb_wc(cs, &wc, (uchar*) b, (uchar*) e)) <= 0)
+    if ((mb_len= my_ci_mb_wc(cs, &wc, (uchar*) b, (uchar*) e)) <= 0)
     {
       mb_len= 1; /* Let's think a wrong sequence takes 1 dysplay cell */
       b++;
@@ -1383,7 +1374,7 @@ int my_mb_ctype_mb(CHARSET_INFO *cs, int *ctype,
                    const uchar *s, const uchar *e)
 {
   my_wc_t wc;
-  int res= cs->cset->mb_wc(cs, &wc, s, e);
+  int res= my_ci_mb_wc(cs, &wc, s, e);
   if (res <= 0 || wc > 0xFFFF)
     *ctype= 0;
   else
