@@ -1,3 +1,18 @@
+/* Copyright (c) 2016, 2020, MariaDB Corporation.
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
+
 #include <my_global.h>
 #include <string.h>
 #include <m_ctype.h>
@@ -895,7 +910,7 @@ int json_read_keyname_chr(json_engine_t *j)
     case S_QUOTE:
       for (;;)  /* Skip spaces until ':'. */
       {
-        if ((c_len= json_next_char(&j->s)) > 0)
+        if ((c_len= json_next_char(&j->s) > 0))
         {
           if (j->s.c_next == ':')
           {
@@ -933,7 +948,6 @@ int json_read_value(json_engine_t *j)
 {
   int t_next, c_len, res;
 
-  j->value_type= JSON_VALUE_UNINITALIZED;
   if (j->state == JST_KEY)
   {
     while (json_read_keyname_chr(j) == 0) {}
@@ -1511,8 +1525,7 @@ int json_append_ascii(CHARSET_INFO *json_cs,
   while (ascii < ascii_end)
   {
     int c_len;
-    if ((c_len= json_cs->cset->wc_mb(json_cs, (my_wc_t) *ascii,
-                                     json, json_end)) > 0)
+    if ((c_len= my_ci_wc_mb(json_cs, (my_wc_t) *ascii, json, json_end)) > 0)
     {
       json+= c_len;
       ascii++;
@@ -1538,7 +1551,7 @@ int json_unescape(CHARSET_INFO *json_cs,
   while (json_read_string_const_chr(&s) == 0)
   {
     int c_len;
-    if ((c_len= res_cs->cset->wc_mb(res_cs, s.c_next, res, res_end)) > 0)
+    if ((c_len= my_ci_wc_mb(res_cs, s.c_next, res, res_end)) > 0)
     {
       res+= c_len;
       continue;
@@ -1549,7 +1562,7 @@ int json_unescape(CHARSET_INFO *json_cs,
         Result charset doesn't support the json's character.
         Let's replace it with the '?' symbol.
       */
-      if ((c_len= res_cs->cset->wc_mb(res_cs, '?', res, res_end)) > 0)
+      if ((c_len= my_ci_wc_mb(res_cs, '?', res, res_end)) > 0)
       {
         res+= c_len;
         continue;
@@ -1608,14 +1621,14 @@ int json_escape(CHARSET_INFO *str_cs,
   {
     my_wc_t c_chr;
     int c_len;
-    if ((c_len= str_cs->cset->mb_wc(str_cs, &c_chr, str, str_end)) > 0)
+    if ((c_len= my_ci_mb_wc(str_cs, &c_chr, str, str_end)) > 0)
     {
       enum json_esc_char_classes c_class;
       
       str+= c_len;
       if (c_chr >= 0x60 || (c_class= json_escape_chr_map[c_chr]) == ESC_)
       {
-        if ((c_len= json_cs->cset->wc_mb(json_cs, c_chr, json, json_end)) > 0)
+        if ((c_len= my_ci_wc_mb(json_cs, c_chr, json, json_end)) > 0)
         {
           json+= c_len;
           continue;
@@ -1630,9 +1643,8 @@ int json_escape(CHARSET_INFO *str_cs,
         c_class= ESC_U;
       }
 
-      if ((c_len= json_cs->cset->wc_mb(json_cs, '\\', json, json_end)) <= 0 ||
-          (c_len= json_cs->cset->wc_mb(json_cs,
-                                       (c_class == ESC_BS) ? c_chr : c_class,
+      if ((c_len= my_ci_wc_mb(json_cs, '\\', json, json_end)) <= 0 ||
+          (c_len= my_ci_wc_mb(json_cs, (c_class == ESC_BS) ? c_chr : c_class,
                                        json+= c_len, json_end)) <= 0)
       {
         /* JSON buffer is depleted. */
@@ -1754,6 +1766,107 @@ int json_get_path_next(json_engine_t *je, json_path_t *p)
 }
 
 
+int json_path_parts_compare(
+    const json_path_step_t *a, const json_path_step_t *a_end,
+    const json_path_step_t *b, const json_path_step_t *b_end,
+    enum json_value_types vt)
+{
+  int res, res2;
+
+  while (a <= a_end)
+  {
+    if (b > b_end)
+    {
+      while (vt != JSON_VALUE_ARRAY &&
+             (a->type & JSON_PATH_ARRAY_WILD) == JSON_PATH_ARRAY &&
+             a->n_item == 0)
+      {
+        if (++a > a_end)
+          return 0;
+      }
+      return -2;
+    }
+
+    DBUG_ASSERT((b->type & (JSON_PATH_WILD | JSON_PATH_DOUBLE_WILD)) == 0);
+
+    
+    if (a->type & JSON_PATH_ARRAY)
+    {
+      if (b->type & JSON_PATH_ARRAY)
+      {
+        if ((a->type & JSON_PATH_WILD) || a->n_item == b->n_item)
+          goto step_fits;
+        goto step_failed;
+      }
+      if ((a->type & JSON_PATH_WILD) == 0 && a->n_item == 0)
+        goto step_fits_autowrap;
+      goto step_failed;
+    }
+    else /* JSON_PATH_KEY */
+    {
+      if (!(b->type & JSON_PATH_KEY))
+        goto step_failed;
+    
+      if (!(a->type & JSON_PATH_WILD) &&
+          (a->key_end - a->key != b->key_end - b->key ||
+           memcmp(a->key, b->key, a->key_end - a->key) != 0))
+        goto step_failed;
+
+      goto step_fits;
+    }
+step_failed:
+    if (!(a->type & JSON_PATH_DOUBLE_WILD))
+      return -1;
+    b++;
+    continue;
+
+step_fits:
+    b++;
+    if (!(a->type & JSON_PATH_DOUBLE_WILD))
+    {
+      a++;
+      continue;
+    }
+
+    /* Double wild handling needs recursions. */
+    res= json_path_parts_compare(a+1, a_end, b, b_end, vt);
+    if (res == 0)
+      return 0;
+
+    res2= json_path_parts_compare(a, a_end, b, b_end, vt);
+
+    return (res2 >= 0) ? res2 : res;
+
+step_fits_autowrap:
+    if (!(a->type & JSON_PATH_DOUBLE_WILD))
+    {
+      a++;
+      continue;
+    }
+
+    /* Double wild handling needs recursions. */
+    res= json_path_parts_compare(a+1, a_end, b+1, b_end, vt);
+    if (res == 0)
+      return 0;
+
+    res2= json_path_parts_compare(a, a_end, b+1, b_end, vt);
+
+    return (res2 >= 0) ? res2 : res;
+
+  }
+
+  return b <= b_end;
+}
+
+
+int json_path_compare(const json_path_t *a, const json_path_t *b,
+                      enum json_value_types vt)
+{
+  return json_path_parts_compare(a->steps+1, a->last_step,
+                                 b->steps+1, b->last_step, vt);
+}
+
+
 static enum json_types smart_read_value(json_engine_t *je,
                                         const char **value, int *value_len)
 {
